from __future__ import absolute_import, division

from collections import defaultdict
import math, os, random, re, urllib

from Orange.orng import orngServerFiles as osf
import statc

from . import obiGene as ge, obiGO as go, obiKEGG as kg, obiProb as op, obiTaxonomy

mirnafile = osf.localpath_download('miRNA','miRNA.txt')
premirnafile = osf.localpath_download('miRNA','premiRNA.txt')

################################################################################################################
################################################################################################################

def __build_lib(filename, labels=True, MATtoPRE=True, ACCtoID=True, clust=False):
    """
    build_lib() function takes as input a filename
    and gives as output some variables there will be used in
    the module.
    """
    content = [l.rstrip() for l in open(filename).readlines()][1:]
    to_return = []
    
    ids = [l.split('\t')[0] for l in content]
    to_return.append(ids)
    
    if labels: 
        to_return.append(list(set(i.split('-')[0] for i in ids)))
    
    elements = [line.rstrip().split('\t') for line in content] 
    to_return.append(dict((elem[0],elem[1:]) for elem in elements))
    
    if MATtoPRE:
        to_return.append(dict([(e[0],e[3]) for e in elements]))
    
    if ACCtoID:
        to_return.append(dict([(e[1],e[0]) for e in elements]))
        
    if clust:
        to_return.append(dict([(e[0],e[5]) for e in elements]))
        
    
    return to_return

def open_microcosm(org="mus_musculus", version="v5"):
    """ Open the miRna targets from the EBI microcosm site. 
    """
    import urllib2, zipfile
    import StringIO
    stream = urllib2.urlopen("ftp://ftp.ebi.ac.uk/pub/databases/microcosm/{version}/arch.{version}.txt.{org}.zip".format(org=org, version=version))
    contents = stream.read()
    stream = StringIO.StringIO(contents)
    return stream
    
def parse_targets_microcosm_v5(file, max_pvalue=None, min_score=None):
    """ Parse miRna targets (version 'v5.txt' format file from microcosm) 
    """
    
    ids = set() #mirna ids
    labels = set() # org codes (i.e hsa, mmu ..)
    #["mirna accession", "seq", "mirna id", "targets"]
    mirna_lib = defaultdict(lambda: ["", "", "", set()])
    mat_to_pre = {}
    if isinstance(file, basestring):
        file = open(file, "rb")
    import csv
    reader = csv.reader(file, delimiter="\t")
    for line in reader:
        if not line or line[0].startswith("##"):
            continue
        
        seq = line[1]
        org = seq.split("-", 1)[0] #SEQ
        ids.add(seq)
        labels.add(org)
        
        mirna_lib[seq] #In case some mirna dont have any targets
        
        if max_pvalue is not None:
            if float(line[10]) > max_pvalue:
                continue
        if min_score is not None:
            if float(line[9]) < min_score:
                continue
            
        mirna_lib[seq][-1].add(line[11]) #TRANSCRIPT_ID
        
    for key, value in mirna_lib.iteritems():
        value[-1] = ",".join(sorted(value[-1]) or ["None"])
        
    ids = sorted(ids)
    labels = sorted(labels)
    mirna_lib = dict(mirna_lib)
    return ids, labels, mirna_lib, {}, {}


def load_miRNA_microCosm(org="mus_musculus", max_pvalue=None, min_score=None):
    """ Load miRNA's from microcosm into the global scope (currently
    only Mus musculus is supported)
    
    """
    global IDs, LABELS, miRNA_lib, mat_toPre, ACCtoID
    global preIDs, premiRNA_lib, preACCtoID, clusters
    global num_toClusters,  clusters_toNum
    
    file = osf.localpath_download("miRNA", "v5.txt.{org}".format(org=org))
    [IDs, LABELS, miRNA_lib, mat_toPre, ACCtoID] = parse_targets_microcosm_v5(file,
                                max_pvalue=max_pvalue, min_score=min_score)
    [preIDs, premiRNA_lib, preACCtoID, clusters] = [], {}, {}, {}
    num_toClusters, clusters_toNum = {}, {}

    
load_miRNA = load_miRNA_microCosm


def load_miRNA_TargetScan():
    """ This loads miRNAs from miRBase and targets from TargetScan.
    Will also load pre-miRNAs
    """
    global IDs, LABELS, miRNA_lib, mat_toPre, ACCtoID
    global preIDs, premiRNA_lib, preACCtoID, clusters
    global num_toClusters,  clusters_toNum
    
    [IDs, LABELS, miRNA_lib, mat_toPre, ACCtoID] = __build_lib(mirnafile, 1,1,1,0)
    [preIDs, premiRNA_lib, preACCtoID, clusters] = __build_lib(premirnafile,0,0,1,1)
    
    num_toClusters = {}
    clusters_toNum = {}
    n=0
    for k,v in clusters.items():
        if v !='None':
            g = v.split(',')
            g.append(k)        
            group = sorted(g)
            if not(group in num_toClusters.values()):
                 num_toClusters[n] = group
                 for e in group:
                     clusters_toNum[e]=n
                 n += 1
     
### library:
load_miRNA()

#[IDs, LABELS, miRNA_lib, mat_toPre, ACCtoID] = __build_lib(mirnafile, 1,1,1,0)
#[preIDs, premiRNA_lib, preACCtoID, clusters] = __build_lib(premirnafile,0,0,1,1)
#
#num_toClusters = {}
#clusters_toNum = {}
#n=0
#for k,v in clusters.items():
#    if v !='None':
#        g = v.split(',')
#        g.append(k)        
#        group = sorted(g)
#        if not(group in num_toClusters.values()):
#             num_toClusters[n] = group
#             for e in group:
#                 clusters_toNum[e]=n
#             n += 1


fromTaxo = {3702:'ath', 9913:'bta', 6239:'cel', 3055:'cre', 7955:'dre',\
             352472:'ddi', 7227:'dme', 9606:'hsa', 10090:'mmu', 4530:'osa',\
              10116:'rno', 8355:'xla', 4577:'zma'}

toTaxo = dict(zip(fromTaxo.values(), fromTaxo.keys()))


class miRNAException(Exception):
    pass

def ids(taxid=None):
    """
    ids() functions takes an organism identifier (for human can be 9606 or hsa)
    and returns all the miRNAs related to that organism. If there 
    is no argument, it returns all the miRNAs in the library.
    """
    
    if not(taxid):
        return IDs
    else:
        taxid = fromTaxo.get(taxid, taxid)
        
        if (taxid in toTaxo):
            return [e for e in IDs if e.split('-')[0]==taxid]
                
        else:
            raise miRNAException("ids() Error: Check the input value.")

            
        
class mat_miRNA:
    pass

class pre_miRNA:
    pass
 
      
def get_info(objectID,type='mat'):
        """
        get_info() function takes a miRNA identifier as input
        and returns a miRNA object.
        """
        if type == 'mat':
            objectID = re.sub('mir','miR',objectID)
            if objectID in IDs:
                attr = [line.rstrip() for line in open(mirnafile).readlines()][0].split('\t')
            
                to_return = mat_miRNA()
                setattr(to_return, attr[0], objectID)
            
                for n,a in enumerate(attr[1:]):
                    setattr(to_return, a, miRNA_lib[objectID][n])
            
                return to_return
            else:
                raise miRNAException("get_info() Error: Check the input value.")
            
        elif type == 'pre':
            objectID = re.sub('miR','mir',objectID)
            if objectID in preIDs:
                attr = [line.rstrip() for line in open(premirnafile).readlines()][0].split('\t')
            
                to_return = pre_miRNA()
                setattr(to_return, attr[0], objectID)
            
                for n,a in enumerate(attr[1:]):
                    setattr(to_return, a, premiRNA_lib[objectID][n])
                            
                return to_return
            else:
                raise miRNAException("get_info() Error: Check the input value.")
        else:
           raise miRNAException("get_info() Error: Check type value.")

                    
def cluster(clusterID, type='name'):
    """
    cluster() function take a cluster identifier or a premiRNA
    and return the list of premiRNAs clustered together."
    """
    if type=='name':
        if clusterID in clusters:
            return clusters[clusterID]
        else:
            raise miRNAException("cluster() Error: ClusterID not found in premiRNA names.")
    
    elif type=='num':
        if clusterID in num_toClusters:
            return num_toClusters[clusterID]
        else:
            raise miRNAException("cluster() Error: ClusterID not found in clusters' list.")
    else:
        raise miRNAException("cluster() Error: Check the input value.")


def fromACC_toID(accession):
    """
    fromACC_toID() takes a miRNA accession number
    and returns a miRNA id.
    """
    if accession in ACCtoID:
        return ACCtoID[accession]
    if accession in preACCtoID:
        return preACCtoID[accession]
    else:
        print "Accession not found."
        return False
    

def __reverseDict(old_dict):
    """
    switch dictionary: keys <--> values; 
    """
    new_dict = {}
    for k,valuesList in old_dict.items():
        for val in valuesList:
            if val != [] and not(val in new_dict):
                new_dict[val]=[]
            if not(k in new_dict[val]):
                new_dict[val].append(k)
    return new_dict


def get_geneMirnaLib(org=None):
    """
    build dictionary gene:[miRNAs]
    """
    mirnaGenes={}
    for m in ids(org):
        mirnaGenes[m] = get_info(m).targets.split(',')
        
    return __reverseDict(mirnaGenes)


def get_GO(mirna_list, annotations, enrichment=False, pval=0.1, goSwitch=True):
    """
    get_GO() takes as input a list of miRNAs of the organism for which the annotations are defined.
    If goSwitch is False, get_GO() returns a dictionary that has miRNAs as keys and GO IDs as values;
    in the other case it returns a dictionary with GO IDs as keys and miRNAs as values.
    """
    
    from . import obiGene
    genematcher = obiGene.matcher([obiGene.GMGO(annotations.taxid)] + \
        ([obiGene.GMDicty()] if annotations.taxid == "352472"  else []))
    genematcher.set_targets(annotations.geneNames)
    
    mirna_list = list(set(mirna_list))
    mirAnnotations = {}
    
    for m in mirna_list:
        genes = get_info(m).targets.split(',')
        genes = filter(None, map(genematcher.umatch, genes))
        
        if enrichment==False:
            mirna_ann=[]
            for gene in genes:
                gene_annotations = annotations.geneAnnotations[gene]
                for ga in gene_annotations:
                    mirna_ann.append(ga)
            mirAnnotations[m] = list(set([an.GO_ID for an in mirna_ann]))
        elif enrichment==True:
            if len(genes):
                resP = annotations.GetEnrichedTerms(genes,aspect='P')
                resC = annotations.GetEnrichedTerms(genes,aspect='C')
                resF = annotations.GetEnrichedTerms(genes,aspect='F')
                res = {}
                res.update(resP)
                res.update(resC)
                res.update(resF)
                tups = [(pVal,go_id) for go_id, (ge,pVal,ref) in res.items()]
                tups.sort()            
                p_correct = op.FDR([p for p,go_id in tups])            
                mirAnnotations[m] = [tups[i][1] for i, p in enumerate(p_correct) if p < pval]
            else:
                mirAnnotations[m]=[]

    if goSwitch:
        return __reverseDict(mirAnnotations)
    else:
        return mirAnnotations
 


def filter_GO(mirna_goid, annotations, treshold=0.04, reverse=True):    
    """
    filter_GO() takes as input a dictionary like {mirna:[list of GO_IDs]} and
    remove the most common GO IDs in each list using the TF-IDF criterion.
    """       
    uniqGO = list(set(reduce(lambda x,y: x+y, mirna_goid.values())))        
    
    goIDF = {}    
    for n,go in enumerate(uniqGO):        
        goIDF[go] = np.log(len(mirna_goid)/len(filter(lambda x: go in x, mirna_goid.values())))        

    data = []
    new_dict={}
    for m,goList in mirna_goid.items():
        TF_IDF ={}
        genes = get_info(m).targets.split(',')
        mirnaAnnotations = [annotations.GetAnnotatedTerms(g).keys() for g in genes]
        for go in goList:
            TF = len(filter(lambda x: go in x, mirnaAnnotations))/len(genes)
            TF_IDF[go] = TF*goIDF[go]
        if treshold:
            t = treshold
        else:
            t=0
        new_dict[m] = filter(lambda x: TF_IDF[x] > t, goList)   
        #data.append(TF_IDF.values())
       
#    if treshold:
#        data = sorted(reduce(lambda x,y: x+y, data))    
#        tresholds = filter(lambda x: x[1]>treshold[0] and x[1]<treshold[1], [(d,statc.percentileofscore(data, d)) for d in list(set(data))])
#        t = np.mean([t[0] for t in tresholds])
#        print t
#    else:
#        t=0
#    
#    new_dict={}     
#    for m,goList in mirna_goid.items():       
#        new_dict[m] = filter(lambda x: TF_IDF[x] > t, goList) 
    
    if reverse:
        return __reverseDict(new_dict)
    else:
        return new_dict



def get_pathways(mirna_list, organism='hsa', enrichment=False, pVal=0.1, pathSwitch=True):
    """
    get_pathways() takes as input a list of miRNAs and returns a dictionary that has miRNAs as keys
    and pathways IDs as values; if the switch is set on True, 
    it returns a dictionary with pathways IDs as keys and miRNAs as values.
    """
    gmkegg = ge.GMKEGG(organism)
    org = kg.KEGGOrganism(organism)
    gmkegg.set_targets(org.get_genes())     
    
    genes = list(set(reduce(lambda x,y: x+y,[get_info(m).targets.split(',') for m in mirna_list])))
    keggNames = dict([(g,gmkegg.umatch(g)) for g in genes if gmkegg.umatch(g)])
        
    mirnaPathways = {}
    for m in mirna_list:
        kegg_genes = [keggNames[g] for g in get_info(m).targets.split(',') if g in keggNames]
        if enrichment:
            mirnaPathways[m] = [path_id for path_id,(geneList,p,geneNum) in org.get_enriched_pathways_by_genes(kegg_genes).items() if p < pVal]
        else:
            paths = filter(None,[list(org.get_pathways_by_genes([k])) for k in kegg_genes])                   
            if paths:
                mirnaPathways[m] = list(set(reduce(lambda x,y: x+y,paths)))
            else:
                mirnaPathways[m] = []
    
    if pathSwitch:
        return __reverseDict(mirnaPathways)
    else:
        return mirnaPathways
        

def removeOldMirnas(mirna_list, getOnlyMature=False):
    """
    removeOldMirnas() takes a list of miRNAs as input and
    divides them in two lists, accordin if they're still present
    on miRBase or not.
    """
    old = []
    for m in mirna_list:
        try:
            mat = get_info(m)
        except Exception:
            if getOnlyMature:
                old.append(m)
            else:                
                try:
                    pre = get_info(m,type='pre')
                except Exception:
                    old.append(m)
    return [old, filter(lambda x: not(x in old), mirna_list)]
    
#######################











      



# Copyright (c) 1999-2000 Gary Strangman; All Rights Reserved.
#
# This software is distributable under the terms of the GNU
# General Public License (GPL) v2, the text of which can be found at
# http://www.gnu.org/copyleft/gpl.html. Installing, importing or otherwise
# using this module constitutes acceptance of the terms of this License.
#
# Disclaimer
# 
# This software is provided "as-is".  There are no expressed or implied
# warranties of any kind, including, but not limited to, the warranties
# of merchantability and fittness for a given application.  In no event
# shall Gary Strangman be liable for any direct, indirect, incidental,
# special, exemplary or consequential damages (including, but not limited
# to, loss of use, data or profits, or business interruption) however
# caused and on any theory of liability, whether in contract, strict
# liability or tort (including negligence or otherwise) arising in any way
# out of the use of this software, even if advised of the possibility of
# such damage.
#
# Comments and/or additions are welcome (send e-mail to:
# strang@nmr.mgh.harvard.edu).
# 
"""
pstat.py module

#################################################
#######  Written by:  Gary Strangman  ###########
#######  Last modified:  Jun 29, 2001 ###########
#################################################

This module provides some useful list and array manipulation routines
modeled after those found in the |Stat package by Gary Perlman, plus a
number of other useful list/file manipulation functions.  The list-based
functions include:

      abut (source,*args)
      simpleabut (source, addon)
      colex (listoflists,cnums)
      collapse (listoflists,keepcols,collapsecols,fcn1=None,fcn2=None,cfcn=None)
      dm (listoflists,criterion)
      flat (l)
      linexand (listoflists,columnlist,valuelist)
      linexor (listoflists,columnlist,valuelist)
      linedelimited (inlist,delimiter)
      lineincols (inlist,colsize) 
      lineincustcols (inlist,colsizes)
      list2string (inlist)
      makelol(inlist)
      makestr(x)
      printcc (lst,extra=2)
      printincols (listoflists,colsize)
      pl (listoflists)
      printl(listoflists)
      replace (lst,oldval,newval)
      recode (inlist,listmap,cols='all')
      remap (listoflists,criterion)
      roundlist (inlist,num_digits_to_round_floats_to)
      sortby(listoflists,sortcols)
      unique (inlist)
      duplicates(inlist)
      writedelimited (listoflists, delimiter, file, writetype='w')

Some of these functions have alternate versions which are defined only if
Numeric (NumPy) can be imported.  These functions are generally named as
above, with an 'a' prefix.

      aabut (source, *args)
      acolex (a,indices,axis=1)
      acollapse (a,keepcols,collapsecols,sterr=0,ns=0)
      adm (a,criterion)
      alinexand (a,columnlist,valuelist)
      alinexor (a,columnlist,valuelist)
      areplace (a,oldval,newval)
      arecode (a,listmap,col='all')
      arowcompare (row1, row2)
      arowsame (row1, row2)
      asortrows(a,axis=0)
      aunique(inarray)
      aduplicates(inarray)

Currently, the code is all but completely un-optimized.  In many cases, the
array versions of functions amount simply to aliases to built-in array
functions/methods.  Their inclusion here is for function name consistency.
"""

## CHANGE LOG:
## ==========
## 01-11-15 ... changed list2string() to accept a delimiter
## 01-06-29 ... converted exec()'s to eval()'s to make compatible with Py2.1
## 01-05-31 ... added duplicates() and aduplicates() functions
## 00-12-28 ... license made GPL, docstring and import requirements
## 99-11-01 ... changed version to 0.3
## 99-08-30 ... removed get, getstrings, put, aget, aput (into io.py)
## 03/27/99 ... added areplace function, made replace fcn recursive
## 12/31/98 ... added writefc function for ouput to fixed column sizes
## 12/07/98 ... fixed import problem (failed on collapse() fcn)
##              added __version__ variable (now 0.2)
## 12/05/98 ... updated doc-strings
##              added features to collapse() function
##              added flat() function for lists
##              fixed a broken asortrows() 
## 11/16/98 ... fixed minor bug in aput for 1D arrays
##
## 11/08/98 ... fixed aput to output large arrays correctly

from __future__ import absolute_import

import string, copy
from types import *

__version__ = 0.4

###===========================  LIST FUNCTIONS  ==========================
###
### Here are the list functions, DEFINED FOR ALL SYSTEMS.
### Array functions (for NumPy-enabled computers) appear below.
###

def abut (source,*args):
    """
Like the |Stat abut command.  It concatenates two lists side-by-side
and returns the result.  '2D' lists are also accomodated for either argument
(source or addon).  CAUTION:  If one list is shorter, it will be repeated
until it is as long as the longest list.  If this behavior is not desired,
use pstat.simpleabut().

Usage:   abut(source, args)   where args=any # of lists
Returns: a list of lists as long as the LONGEST list past, source on the
         'left', lists in <args> attached consecutively on the 'right'
"""

    if type(source) not in [ListType,TupleType]:
        source = [source]
    for addon in args:
        if type(addon) not in [ListType,TupleType]:
            addon = [addon]
        if len(addon) < len(source):                # is source list longer?
            if len(source) % len(addon) == 0:        # are they integer multiples?
                repeats = len(source)/len(addon)    # repeat addon n times
                origadd = copy.deepcopy(addon)
                for i in range(repeats-1):
                    addon = addon + origadd
            else:
                repeats = len(source)/len(addon)+1  # repeat addon x times,
                origadd = copy.deepcopy(addon)      #    x is NOT an integer
                for i in range(repeats-1):
                    addon = addon + origadd
                    addon = addon[0:len(source)]
        elif len(source) < len(addon):                # is addon list longer?
            if len(addon) % len(source) == 0:        # are they integer multiples?
                repeats = len(addon)/len(source)    # repeat source n times
                origsour = copy.deepcopy(source)
                for i in range(repeats-1):
                    source = source + origsour
            else:
                repeats = len(addon)/len(source)+1  # repeat source x times,
                origsour = copy.deepcopy(source)    #   x is NOT an integer
                for i in range(repeats-1):
                    source = source + origsour
                source = source[0:len(addon)]

        source = simpleabut(source,addon)
    return source


def simpleabut (source, addon):
    """
Concatenates two lists as columns and returns the result.  '2D' lists
are also accomodated for either argument (source or addon).  This DOES NOT
repeat either list to make the 2 lists of equal length.  Beware of list pairs
with different lengths ... the resulting list will be the length of the
FIRST list passed.

Usage:   simpleabut(source,addon)  where source, addon=list (or list-of-lists)
Returns: a list of lists as long as source, with source on the 'left' and
                 addon on the 'right'
"""
    if type(source) not in [ListType,TupleType]:
        source = [source]
    if type(addon) not in [ListType,TupleType]:
        addon = [addon]
    minlen = min(len(source),len(addon))
    list = copy.deepcopy(source)                # start abut process
    if type(source[0]) not in [ListType,TupleType]:
        if type(addon[0]) not in [ListType,TupleType]:
            for i in range(minlen):
                list[i] = [source[i]] + [addon[i]]        # source/addon = column
        else:
            for i in range(minlen):
                list[i] = [source[i]] + addon[i]        # addon=list-of-lists
    else:
        if type(addon[0]) not in [ListType,TupleType]:
            for i in range(minlen):
                list[i] = source[i] + [addon[i]]        # source=list-of-lists
        else:
            for i in range(minlen):
                list[i] = source[i] + addon[i]        # source/addon = list-of-lists
    source = list
    return source


def colex (listoflists,cnums):
    """
Extracts from listoflists the columns specified in the list 'cnums'
(cnums can be an integer, a sequence of integers, or a string-expression that
corresponds to a slice operation on the variable x ... e.g., 'x[3:]' will colex
columns 3 onward from the listoflists).

Usage:   colex (listoflists,cnums)
Returns: a list-of-lists corresponding to the columns from listoflists
         specified by cnums, in the order the column numbers appear in cnums
"""
    global index
    column = 0
    if type(cnums) in [ListType,TupleType]:   # if multiple columns to get
        index = cnums[0]
        column = map(lambda x: x[index], listoflists)
        for col in cnums[1:]:
            index = col
            column = abut(column,map(lambda x: x[index], listoflists))
    elif type(cnums) == StringType:              # if an 'x[3:]' type expr.
        evalstring = 'map(lambda x: x'+cnums+', listoflists)'
        column = eval(evalstring)
    else:                                     # else it's just 1 col to get
        index = cnums
        column = map(lambda x: x[index], listoflists)
    return column


def collapse (listoflists,keepcols,collapsecols,fcn1=None,fcn2=None,cfcn=None):
     """
Averages data in collapsecol, keeping all unique items in keepcols
(using unique, which keeps unique LISTS of column numbers), retaining the
unique sets of values in keepcols, the mean for each.  Setting fcn1
and/or fcn2 to point to a function rather than None (e.g., stats.sterr, len)
will append those results (e.g., the sterr, N) after each calculated mean.
cfcn is the collapse function to apply (defaults to mean, defined here in the
pstat module to avoid circular imports with stats.py, but harmonicmean or
others could be passed).

Usage:    collapse (listoflists,keepcols,collapsecols,fcn1=None,fcn2=None,cfcn=None)
Returns: a list of lists with all unique permutations of entries appearing in
     columns ("conditions") specified by keepcols, abutted with the result of
     cfcn (if cfcn=None, defaults to the mean) of each column specified by
     collapsecols.
"""
     def collmean (inlist):
         s = 0
         for item in inlist:
             s = s + item
         return s/float(len(inlist))

     if type(keepcols) not in [ListType,TupleType]:
         keepcols = [keepcols]
     if type(collapsecols) not in [ListType,TupleType]:
         collapsecols = [collapsecols]
     if cfcn == None:
         cfcn = collmean
     if keepcols == []:
         means = [0]*len(collapsecols)
         for i in range(len(collapsecols)):
             avgcol = colex(listoflists,collapsecols[i])
             means[i] = cfcn(avgcol)
             if fcn1:
                 try:
                     test = fcn1(avgcol)
                 except:
                     test = 'N/A'
                     means[i] = [means[i], test]
             if fcn2:
                 try:
                     test = fcn2(avgcol)
                 except:
                     test = 'N/A'
                 try:
                     means[i] = means[i] + [len(avgcol)]
                 except TypeError:
                     means[i] = [means[i],len(avgcol)]
         return means
     else:
         values = colex(listoflists,keepcols)
         uniques = unique(values)
         uniques.sort()
         newlist = []
         if type(keepcols) not in [ListType,TupleType]:  keepcols = [keepcols]
         for item in uniques:
             if type(item) not in [ListType,TupleType]:  item =[item]
             tmprows = linexand(listoflists,keepcols,item)
             for col in collapsecols:
                 avgcol = colex(tmprows,col)
                 item.append(cfcn(avgcol))
                 if fcn1 <> None:
                     try:
                         test = fcn1(avgcol)
                     except:
                         test = 'N/A'
                     item.append(test)
                 if fcn2 <> None:
                     try:
                         test = fcn2(avgcol)
                     except:
                         test = 'N/A'
                     item.append(test)
                 newlist.append(item)
         return newlist


def dm (listoflists,criterion):
    """
Returns rows from the passed list of lists that meet the criteria in
the passed criterion expression (a string as a function of x; e.g., 'x[3]>=9'
will return all rows where the 4th column>=9 and "x[2]=='N'" will return rows
with column 2 equal to the string 'N').

Usage:   dm (listoflists, criterion)
Returns: rows from listoflists that meet the specified criterion.
"""
    function = 'filter(lambda x: '+criterion+',listoflists)'
    lines = eval(function)
    return lines


def flat(l):
    """
Returns the flattened version of a '2D' list.  List-correlate to the a.flat()
method of NumPy arrays.

Usage:    flat(l)
"""
    newl = []
    for i in range(len(l)):
        for j in range(len(l[i])):
            newl.append(l[i][j])
    return newl


def linexand (listoflists,columnlist,valuelist):
    """
Returns the rows of a list of lists where col (from columnlist) = val
(from valuelist) for EVERY pair of values (columnlist[i],valuelists[i]).
len(columnlist) must equal len(valuelist).

Usage:   linexand (listoflists,columnlist,valuelist)
Returns: the rows of listoflists where columnlist[i]=valuelist[i] for ALL i
"""
    if type(columnlist) not in [ListType,TupleType]:
        columnlist = [columnlist]
    if type(valuelist) not in [ListType,TupleType]:
        valuelist = [valuelist]
    criterion = ''
    for i in range(len(columnlist)):
        if type(valuelist[i])==StringType:
            critval = '\'' + valuelist[i] + '\''
        else:
            critval = str(valuelist[i])
        criterion = criterion + ' x['+str(columnlist[i])+']=='+critval+' and'
    criterion = criterion[0:-3]         # remove the "and" after the last crit
    function = 'filter(lambda x: '+criterion+',listoflists)'
    lines = eval(function)
    return lines


def linexor (listoflists,columnlist,valuelist):
    """
Returns the rows of a list of lists where col (from columnlist) = val
(from valuelist) for ANY pair of values (colunmlist[i],valuelist[i[).
One value is required for each column in columnlist.  If only one value
exists for columnlist but multiple values appear in valuelist, the
valuelist values are all assumed to pertain to the same column.

Usage:   linexor (listoflists,columnlist,valuelist)
Returns: the rows of listoflists where columnlist[i]=valuelist[i] for ANY i
"""
    if type(columnlist) not in [ListType,TupleType]:
        columnlist = [columnlist]
    if type(valuelist) not in [ListType,TupleType]:
        valuelist = [valuelist]
    criterion = ''
    if len(columnlist) == 1 and len(valuelist) > 1:
        columnlist = columnlist*len(valuelist)
    for i in range(len(columnlist)):          # build an exec string
        if type(valuelist[i])==StringType:
            critval = '\'' + valuelist[i] + '\''
        else:
            critval = str(valuelist[i])
        criterion = criterion + ' x['+str(columnlist[i])+']=='+critval+' or'
    criterion = criterion[0:-2]         # remove the "or" after the last crit
    function = 'filter(lambda x: '+criterion+',listoflists)'
    lines = eval(function)
    return lines


def linedelimited (inlist,delimiter):
    """
Returns a string composed of elements in inlist, with each element
separated by 'delimiter.'  Used by function writedelimited.  Use '\t'
for tab-delimiting.

Usage:   linedelimited (inlist,delimiter)
"""
    outstr = ''
    for item in inlist:
        if type(item) <> StringType:
            item = str(item)
        outstr = outstr + item + delimiter
    outstr = outstr[0:-1]
    return outstr


def lineincols (inlist,colsize):
    """
Returns a string composed of elements in inlist, with each element
right-aligned in columns of (fixed) colsize.

Usage:   lineincols (inlist,colsize)   where colsize is an integer
"""
    outstr = ''
    for item in inlist:
        if type(item) <> StringType:
            item = str(item)
        size = len(item)
        if size <= colsize:
            for i in range(colsize-size):
                outstr = outstr + ' '
            outstr = outstr + item
        else:
            outstr = outstr + item[0:colsize+1]
    return outstr


def lineincustcols (inlist,colsizes):
    """
Returns a string composed of elements in inlist, with each element
right-aligned in a column of width specified by a sequence colsizes.  The
length of colsizes must be greater than or equal to the number of columns
in inlist.

Usage:   lineincustcols (inlist,colsizes)
Returns: formatted string created from inlist
"""
    outstr = ''
    for i in range(len(inlist)):
        if type(inlist[i]) <> StringType:
            item = str(inlist[i])
        else:
            item = inlist[i]
        size = len(item)
        if size <= colsizes[i]:
            for j in range(colsizes[i]-size):
                outstr = outstr + ' '
            outstr = outstr + item
        else:
            outstr = outstr + item[0:colsizes[i]+1]
    return outstr


def list2string (inlist,delimit=' '):
    """
Converts a 1D list to a single long string for file output, using
the string.join function.

Usage:   list2string (inlist,delimit=' ')
Returns: the string created from inlist
"""
    stringlist = map(makestr,inlist)
    return string.join(stringlist,delimit)


def makelol(inlist):
    """
Converts a 1D list to a 2D list (i.e., a list-of-lists).  Useful when you
want to use put() to write a 1D list one item per line in the file.

Usage:   makelol(inlist)
Returns: if l = [1,2,'hi'] then returns [[1],[2],['hi']] etc.
"""
    x = []
    for item in inlist:
        x.append([item])
    return x


def makestr (x):
    if type(x) <> StringType:
        x = str(x)
    return x


def printcc (lst,extra=2):
    """
Prints a list of lists in columns, customized by the max size of items
within the columns (max size of items in col, plus 'extra' number of spaces).
Use 'dashes' or '\\n' in the list-of-lists to print dashes or blank lines,
respectively.

Usage:   printcc (lst,extra=2)
Returns: None
"""
    if type(lst[0]) not in [ListType,TupleType]:
        lst = [lst]
    rowstokill = []
    list2print = copy.deepcopy(lst)
    for i in range(len(lst)):
        if lst[i] == ['\n'] or lst[i]=='\n' or lst[i]=='dashes' or lst[i]=='' or lst[i]==['']:
            rowstokill = rowstokill + [i]
    rowstokill.reverse()   # delete blank rows from the end
    for row in rowstokill:
        del list2print[row]
    maxsize = [0]*len(list2print[0])
    for col in range(len(list2print[0])):
        items = colex(list2print,col)
        items = map(makestr,items)
        maxsize[col] = max(map(len,items)) + extra
    for row in lst:
        if row == ['\n'] or row == '\n' or row == '' or row == ['']:
            print
        elif row == ['dashes'] or row == 'dashes':
            dashes = [0]*len(maxsize)
            for j in range(len(maxsize)):
                dashes[j] = '-'*(maxsize[j]-2)
            print lineincustcols(dashes,maxsize)
        else:
            print lineincustcols(row,maxsize)
    return None


def printincols (listoflists,colsize):
    """
Prints a list of lists in columns of (fixed) colsize width, where
colsize is an integer.

Usage:   printincols (listoflists,colsize)
Returns: None
"""
    for row in listoflists:
        print lineincols(row,colsize)
    return None


def pl (listoflists):
    """
Prints a list of lists, 1 list (row) at a time.

Usage:   pl(listoflists)
Returns: None
"""
    for row in listoflists:
        if row[-1] == '\n':
            print row,
        else:
            print row
    return None


def printl(listoflists):
    """Alias for pl."""
    pl(listoflists)
    return


def replace (inlst,oldval,newval):
    """
Replaces all occurrences of 'oldval' with 'newval', recursively.

Usage:   replace (inlst,oldval,newval)
"""
    lst = inlst*1
    for i in range(len(lst)):
        if type(lst[i]) not in [ListType,TupleType]:
            if lst[i]==oldval: lst[i]=newval
        else:
            lst[i] = replace(lst[i],oldval,newval)
    return lst


def recode (inlist,listmap,cols=None):
    """
Changes the values in a list to a new set of values (useful when
you need to recode data from (e.g.) strings to numbers.  cols defaults
to None (meaning all columns are recoded).

Usage:   recode (inlist,listmap,cols=None)  cols=recode cols, listmap=2D list
Returns: inlist with the appropriate values replaced with new ones
"""
    lst = copy.deepcopy(inlist)
    if cols != None:
        if type(cols) not in [ListType,TupleType]:
            cols = [cols]
        for col in cols:
            for row in range(len(lst)):
                try:
                    idx = colex(listmap,0).index(lst[row][col])
                    lst[row][col] = listmap[idx][1]
                except ValueError:
                    pass
    else:
        for row in range(len(lst)):
            for col in range(len(lst)):
                try:
                    idx = colex(listmap,0).index(lst[row][col])
                    lst[row][col] = listmap[idx][1]
                except ValueError:
                    pass
    return lst


def remap (listoflists,criterion):
    """
Remaps values in a given column of a 2D list (listoflists).  This requires
a criterion as a function of 'x' so that the result of the following is
returned ... map(lambda x: 'criterion',listoflists).  

Usage:   remap(listoflists,criterion)    criterion=string
Returns: remapped version of listoflists
"""
    function = 'map(lambda x: '+criterion+',listoflists)'
    lines = eval(function)
    return lines


def roundlist (inlist,digits):
    """
Goes through each element in a 1D or 2D inlist, and applies the following
function to all elements of FloatType ... round(element,digits).

Usage:   roundlist(inlist,digits)
Returns: list with rounded floats
"""
    if type(inlist[0]) in [IntType, FloatType]:
        inlist = [inlist]
    l = inlist*1
    for i in range(len(l)):
        for j in range(len(l[i])):
            if type(l[i][j])==FloatType:
                l[i][j] = round(l[i][j],digits)
    return l


def sortby(listoflists,sortcols):
    """
Sorts a list of lists on the column(s) specified in the sequence
sortcols.

Usage:   sortby(listoflists,sortcols)
Returns: sorted list, unchanged column ordering
"""
    newlist = abut(colex(listoflists,sortcols),listoflists)
    newlist.sort()
    try:
        numcols = len(sortcols)
    except TypeError:
        numcols = 1
    crit = '[' + str(numcols) + ':]'
    newlist = colex(newlist,crit)
    return newlist


def unique (inlist):
    """
Returns all unique items in the passed list.  If the a list-of-lists
is passed, unique LISTS are found (i.e., items in the first dimension are
compared).

Usage:   unique (inlist)
Returns: the unique elements (or rows) in inlist
"""
    uniques = []
    for item in inlist:
        if item not in uniques:
            uniques.append(item)
    return uniques

def duplicates(inlist):
    """
Returns duplicate items in the FIRST dimension of the passed list.

Usage:   duplicates (inlist)
"""
    dups = []
    for i in range(len(inlist)):
        if inlist[i] in inlist[i+1:]:
            dups.append(inlist[i])
    return dups


def nonrepeats(inlist):
    """
Returns items that are NOT duplicated in the first dim of the passed list.

Usage:   nonrepeats (inlist)
"""
    nonrepeats = []
    for i in range(len(inlist)):
        if inlist.count(inlist[i]) == 1:
            nonrepeats.append(inlist[i])
    return nonrepeats


#===================   PSTAT ARRAY FUNCTIONS  =====================
#===================   PSTAT ARRAY FUNCTIONS  =====================
#===================   PSTAT ARRAY FUNCTIONS  =====================
#===================   PSTAT ARRAY FUNCTIONS  =====================
#===================   PSTAT ARRAY FUNCTIONS  =====================
#===================   PSTAT ARRAY FUNCTIONS  =====================
#===================   PSTAT ARRAY FUNCTIONS  =====================
#===================   PSTAT ARRAY FUNCTIONS  =====================
#===================   PSTAT ARRAY FUNCTIONS  =====================
#===================   PSTAT ARRAY FUNCTIONS  =====================
#===================   PSTAT ARRAY FUNCTIONS  =====================
#===================   PSTAT ARRAY FUNCTIONS  =====================
#===================   PSTAT ARRAY FUNCTIONS  =====================
#===================   PSTAT ARRAY FUNCTIONS  =====================
#===================   PSTAT ARRAY FUNCTIONS  =====================
#===================   PSTAT ARRAY FUNCTIONS  =====================

try:                         # DEFINE THESE *ONLY* IF NUMERIC IS AVAILABLE
 import Numeric
 N = Numeric

 def aabut (source, *args):
    """
Like the |Stat abut command.  It concatenates two arrays column-wise
and returns the result.  CAUTION:  If one array is shorter, it will be
repeated until it is as long as the other.

Usage:   aabut (source, args)    where args=any # of arrays
Returns: an array as long as the LONGEST array past, source appearing on the
         'left', arrays in <args> attached on the 'right'.
"""
    if len(source.shape)==1:
        width = 1
        source = N.resize(source,[source.shape[0],width])
    else:
        width = source.shape[1]
    for addon in args:
        if len(addon.shape)==1:
            width = 1
            addon = N.resize(addon,[source.shape[0],width])
        else:
            width = source.shape[1]
        if len(addon) < len(source):
            addon = N.resize(addon,[source.shape[0],addon.shape[1]])
        elif len(source) < len(addon):
            source = N.resize(source,[addon.shape[0],source.shape[1]])
        source = N.concatenate((source,addon),1)
    return source


 def acolex (a,indices,axis=1):
    """
Extracts specified indices (a list) from passed array, along passed
axis (column extraction is default).  BEWARE: A 1D array is presumed to be a
column-array (and that the whole array will be returned as a column).

Usage:   acolex (a,indices,axis=1)
Returns: the columns of a specified by indices
"""
    if type(indices) not in [ListType,TupleType,N.ArrayType]:
        indices = [indices]
    if len(N.shape(a)) == 1:
        cols = N.resize(a,[a.shape[0],1])
    else:
        cols = N.take(a,indices,axis)
    return cols


 def acollapse (a,keepcols,collapsecols,fcn1=None,fcn2=None,cfcn=None):
    """
Averages data in collapsecol, keeping all unique items in keepcols
(using unique, which keeps unique LISTS of column numbers), retaining
the unique sets of values in keepcols, the mean for each.  If stderror or
N of the mean are desired, set either or both parameters to 1.

Usage:   acollapse (a,keepcols,collapsecols,fcn1=None,fcn2=None,cfcn=None)
Returns: unique 'conditions' specified by the contents of columns specified
         by keepcols, abutted with the mean(s) of column(s) specified by
         collapsecols
"""
    def acollmean (inarray):
        return N.sum(N.ravel(inarray))

    if cfcn == None:
        cfcn = acollmean
    if keepcols == []:
        avgcol = acolex(a,collapsecols)
        means = N.sum(avgcol)/float(len(avgcol))
        if fcn1<>None:
            try:
                test = fcn1(avgcol)
            except:
                test = N.array(['N/A']*len(means))
            means = aabut(means,test)
        if fcn2<>None:
            try:
                test = fcn2(avgcol)
            except:
                test = N.array(['N/A']*len(means))
            means = aabut(means,test)
        return means
    else:
        if type(keepcols) not in [ListType,TupleType,N.ArrayType]:
            keepcols = [keepcols]
        values = colex(a,keepcols)   # so that "item" can be appended (below)
        uniques = unique(values)  # get a LIST, so .sort keeps rows intact
        uniques.sort()
        newlist = []
        for item in uniques:
            if type(item) not in [ListType,TupleType,N.ArrayType]:
                item =[item]
            tmprows = alinexand(a,keepcols,item)
            for col in collapsecols:
                avgcol = acolex(tmprows,col)
                item.append(acollmean(avgcol))
                if fcn1<>None:
                    try:
                        test = fcn1(avgcol)
                    except:
                        test = 'N/A'
                    item.append(test)
                if fcn2<>None:
                    try:
                        test = fcn2(avgcol)
                    except:
                        test = 'N/A'
                    item.append(test)
                newlist.append(item)
        try:
            new_a = N.array(newlist)
        except TypeError:
            new_a = N.array(newlist,'O')
        return new_a


 def adm (a,criterion):
    """
Returns rows from the passed list of lists that meet the criteria in
the passed criterion expression (a string as a function of x).

Usage:   adm (a,criterion)   where criterion is like 'x[2]==37'
"""
    function = 'filter(lambda x: '+criterion+',a)'
    lines = eval(function)
    try:
        lines = N.array(lines)
    except:
        lines = N.array(lines,'O')
    return lines


 def isstring(x):
    if type(x)==StringType:
        return 1
    else:
        return 0


 def alinexand (a,columnlist,valuelist):
    """
Returns the rows of an array where col (from columnlist) = val
(from valuelist).  One value is required for each column in columnlist.

Usage:   alinexand (a,columnlist,valuelist)
Returns: the rows of a where columnlist[i]=valuelist[i] for ALL i
"""
    if type(columnlist) not in [ListType,TupleType,N.ArrayType]:
        columnlist = [columnlist]
    if type(valuelist) not in [ListType,TupleType,N.ArrayType]:
        valuelist = [valuelist]
    criterion = ''
    for i in range(len(columnlist)):
        if type(valuelist[i])==StringType:
            critval = '\'' + valuelist[i] + '\''
        else:
            critval = str(valuelist[i])
        criterion = criterion + ' x['+str(columnlist[i])+']=='+critval+' and'
    criterion = criterion[0:-3]         # remove the "and" after the last crit
    return adm(a,criterion)


 def alinexor (a,columnlist,valuelist):
    """
Returns the rows of an array where col (from columnlist) = val (from
valuelist).  One value is required for each column in columnlist.
The exception is if either columnlist or valuelist has only 1 value,
in which case that item will be expanded to match the length of the
other list.

Usage:   alinexor (a,columnlist,valuelist)
Returns: the rows of a where columnlist[i]=valuelist[i] for ANY i
"""
    if type(columnlist) not in [ListType,TupleType,N.ArrayType]:
        columnlist = [columnlist]
    if type(valuelist) not in [ListType,TupleType,N.ArrayType]:
        valuelist = [valuelist]
    criterion = ''
    if len(columnlist) == 1 and len(valuelist) > 1:
        columnlist = columnlist*len(valuelist)
    elif len(valuelist) == 1 and len(columnlist) > 1:
        valuelist = valuelist*len(columnlist)
    for i in range(len(columnlist)):
        if type(valuelist[i])==StringType:
            critval = '\'' + valuelist[i] + '\''
        else:
            critval = str(valuelist[i])
        criterion = criterion + ' x['+str(columnlist[i])+']=='+critval+' or'
    criterion = criterion[0:-2]         # remove the "or" after the last crit
    return adm(a,criterion)


 def areplace (a,oldval,newval):
    """
Replaces all occurrences of oldval with newval in array a.

Usage:   areplace(a,oldval,newval)
"""
    newa = N.not_equal(a,oldval)*a
    return newa+N.equal(a,oldval)*newval


 def arecode (a,listmap,col='all'):
    """
Remaps the values in an array to a new set of values (useful when
you need to recode data from (e.g.) strings to numbers as most stats
packages require.  Can work on SINGLE columns, or 'all' columns at once.

Usage:   arecode (a,listmap,col='all')
Returns: a version of array a where listmap[i][0] = (instead) listmap[i][1]
"""
    ashape = a.shape
    if col == 'all':
        work = a.flat
    else:
        work = acolex(a,col)
        work = work.flat
    for pair in listmap:
        if type(pair[1]) == StringType or work.typecode()=='O' or a.typecode()=='O':
            work = N.array(work,'O')
            a = N.array(a,'O')
            for i in range(len(work)):
                if work[i]==pair[0]:
                    work[i] = pair[1]
            if col == 'all':
                return N.reshape(work,ashape)
            else:
                return N.concatenate([a[:,0:col],work[:,N.NewAxis],a[:,col+1:]],1)
        else:   # must be a non-Object type array and replacement
            work = N.where(N.equal(work,pair[0]),pair[1],work)
            return N.concatenate([a[:,0:col],work[:,N.NewAxis],a[:,col+1:]],1)


 def arowcompare(row1, row2):
    """
Compares two rows from an array, regardless of whether it is an
array of numbers or of python objects (which requires the cmp function).

Usage:   arowcompare(row1,row2)
Returns: an array of equal length containing 1s where the two rows had
         identical elements and 0 otherwise
"""
    if row1.typecode()=='O' or row2.typecode=='O':
        cmpvect = N.logical_not(abs(N.array(map(cmp,row1,row2)))) # cmp fcn gives -1,0,1
    else:
        cmpvect = N.equal(row1,row2)
    return cmpvect


 def arowsame(row1, row2):
    """
Compares two rows from an array, regardless of whether it is an
array of numbers or of python objects (which requires the cmp function).

Usage:   arowsame(row1,row2)
Returns: 1 if the two rows are identical, 0 otherwise.
"""
    cmpval = N.alltrue(arowcompare(row1,row2))
    return cmpval


 def asortrows(a,axis=0):
    """
Sorts an array "by rows".  This differs from the Numeric.sort() function,
which sorts elements WITHIN the given axis.  Instead, this function keeps
the elements along the given axis intact, but shifts them 'up or down'
relative to one another.

Usage:   asortrows(a,axis=0)
Returns: sorted version of a
"""
    if axis != 0:
        a = N.swapaxes(a, axis, 0)
    l = a.tolist()
    l.sort()           # or l.sort(_sort)
    y = N.array(l)
    if axis != 0:
        y = N.swapaxes(y, axis, 0)
    return y


 def aunique(inarray):
    """
Returns unique items in the FIRST dimension of the passed array. Only
works on arrays NOT including string items.

Usage:   aunique (inarray)
"""
    uniques = N.array([inarray[0]])
    if len(uniques.shape) == 1:            # IF IT'S A 1D ARRAY
        for item in inarray[1:]:
            if N.add.reduce(N.equal(uniques,item).flat) == 0:
                try:
                    uniques = N.concatenate([uniques,N.array[N.NewAxis,:]])
                except TypeError:
                    uniques = N.concatenate([uniques,N.array([item])])
    else:                                  # IT MUST BE A 2+D ARRAY
        if inarray.typecode() != 'O':  # not an Object array
            for item in inarray[1:]:
                if not N.sum(N.alltrue(N.equal(uniques,item),1)):
                    try:
                        uniques = N.concatenate( [uniques,item[N.NewAxis,:]] )
                    except TypeError:    # the item to add isn't a list
                        uniques = N.concatenate([uniques,N.array([item])])
                else:
                    pass  # this item is already in the uniques array
        else:   # must be an Object array, alltrue/equal functions don't work
            for item in inarray[1:]:
                newflag = 1
                for unq in uniques:  # NOTE: cmp --> 0=same, -1=<, 1=>
                    test = N.sum(abs(N.array(map(cmp,item,unq))))
                    if test == 0:   # if item identical to any 1 row in uniques
                        newflag = 0 # then not a novel item to add
                        break
                if newflag == 1:
                    try:
                        uniques = N.concatenate( [uniques,item[N.NewAxis,:]] )
                    except TypeError:    # the item to add isn't a list
                        uniques = N.concatenate([uniques,N.array([item])])
    return uniques


 def aduplicates(inarray):
    """
Returns duplicate items in the FIRST dimension of the passed array. Only
works on arrays NOT including string items.

Usage:   aunique (inarray)
"""
    inarray = N.array(inarray)
    if len(inarray.shape) == 1:            # IF IT'S A 1D ARRAY
        dups = []
        inarray = inarray.tolist()
        for i in range(len(inarray)):
            if inarray[i] in inarray[i+1:]:
                dups.append(inarray[i])
        dups = aunique(dups)
    else:                                  # IT MUST BE A 2+D ARRAY
        dups = []
        aslist = inarray.tolist()
        for i in range(len(aslist)):
            if aslist[i] in aslist[i+1:]:
                dups.append(aslist[i])
        dups = unique(dups)
        dups = N.array(dups)
    return dups

except ImportError:    # IF NUMERIC ISN'T AVAILABLE, SKIP ALL arrayfuncs
 pass



"""
===============
obiArrayExpress
===============

A python module for accessing the ArrayExpress web services and database.

`Array Express Archive <http://www.ebi.ac.uk/arrayexpress/>`_ is a database of gene expression experiments that you
can query and download.

Example ::

    >>> # Retrieve the object representing experiment with accession E-TABM-25 
    >>> experiment = ArrayExpressExperiment("E-TABM-25")
    >>> print experiment.accession
    E-TABM-25
    
    >>> print experiment.name
    Transcription profiling of aging in the primate brain
    
    >>> print experiment.species
    ['Pan troglodytes']
    
    >>> print experiment.files
    [{'kind': ...
    
    >>> # Retrieve the data matrix for experiment 'E-MEXP-2917'
    >>> experiment = ArrayExpressExperiment("E-MEXP-2917")
    >>> table = experiment.fgem_to_table() 


Low level Array Express query using REST services::

    >>> from Orange.bioinformatics import obiArrayExpress
    >>> obiArrayExpress.query_experiments(accession='E-MEXP-31')
    {u'experiments': ...
    
    >>> obiArrayExpress.query_experiments(keywords='gliobastoma')
    {u'experiments': ...
    
    >>> obiArrayExpress.query_files(accession='E-MEXP-32', format="xml")
    <xml.etree.ElementTree.ElementTree object ...
   
.. note:: Currently querying ArrayExpress files only works with the xml format.

.. note:: See the documentation of `query_experiments` for a full set of
          parameters that these functions accept.

"""

from __future__ import absolute_import

import os, sys
import urllib2

from Orange.orng import orngServerFiles
import warnings
import posixpath
import shelve
import shutil
import posixpath
import json
from xml.etree.ElementTree import ElementTree
from StringIO import StringIO

from collections import defaultdict
from functools import partial
from contextlib import closing

parse_json = json.load

def parse_xml(stream):
    """ Parse an xml stream into an instance of xml.etree.ElementTree.ElementTree.
    """
    return ElementTree(file=stream) 

# All searchable fields of ArrayExpress (see query_experiments docstring
# for a description of the fields)
ARRAYEXPRESS_FIELDS = \
    ["keywords",
     "accession",
     "array",
     "ef",
     "efv",
     "expdesign",
     "exptype",
     "gxa",
     "pmid",
     "sa",
     "species",
     "expandefo",
     "directsub",
     "assaycount",
     "efcount",
     "samplecount",
     "sacount",
     "rawcount",
     "fgemcount",
     "miamescore",
     "date",
     "wholewords",
    ]
    

class ArrayExpressConnection(object):
    """ A connection to the ArrayExpress. Used to construct a REST query
    and run it.
    
    .. todo:: Implement user login.
    
    """
    
    DEFAULT_ADDRESS = "http://www.ebi.ac.uk/arrayexpress/{format}/v2/"
    DEFAULT_FORMAT = "json"
    DEFAULT_CACHE = orngServerFiles.localpath("ArrayExpress", "ArrayExpressCache.shelve")
    # Order of arguments in the query
    _ARGS_ORDER = ["keywords", "species", "array"]
    
    def __init__(self, address=None, timeout=30, cache=None,
                 username=None, password=None):
        """ Initialize the connection object.
        
        :param address: Address of the ArrayExpress API
        :param timeout: Timeout for the socket connection
        
        .. todo:: Implement user login (see Accessing Private Data in API docs)
        
        """
        self.address = address if address is not None else self.DEFAULT_ADDRESS
        self.timeout = timeout
        self.cache = cache if cache is not None else self.DEFAULT_CACHE
        self.username = username
        self.password = password
        
    def format_query(self, **kwargs):
        """ Format the query arguments.
        
        Example ::
        
            >>> conn.format_query(gxa=True, efcount=(1, 5))
            'efcount=[1 TO 5]&gxa=true'
            
        """
        # Formaters:
        def format_default(val):
            if isinstance(val, basestring):
                return val
            else:
                return "+".join(val)
        def format_species(val):
            return '"%s"' % val.lower()
        def format_gxa(val):
            if val:
                return "true"
            else:
                raise ValueError("gxa={0}".format(val))
        def format_expandefo(val):
            if val:
                return "on"
            else:
                raise ValueError("expandefo={0}".format(val))
        def format_true_false(val):
            return "true" if val else "false"
        def format_interval(val):
            if isinstance(val, tuple):
                return "[{0} TO {1}]".format(*val)
            else:
                raise ValueError("Must be an interval argument (min, max)!")
        def format_date(val):
            # TODO check if val contains a datetime.date object, assert proper format
            return format_interval(val)
        def format_wholewords(val):
            if val:
                return "on"
            else:
                raise ValueError("wholewords={0}".format(val))
        
        formaters = {"species": format_species,
                     "gxa": format_gxa,
                     "expandefo": format_expandefo,
                     "directsub": format_true_false,
                     "assaycount": format_interval,
                     "efcount": format_interval,
                     "samplecount": format_interval,
                     "sacount": format_interval,
                     "rawcount": format_interval,
                     "fgemcount": format_interval,
                     "miamescore": format_interval,
                     "date": format_date,
                     "wholewords": format_wholewords,
                     }
        parts = []
        arg_items = kwargs.items()
        ordered = sorted(arg_items, key=lambda arg: self._ARGS_ORDER.index(arg[0]) \
                         if arg[0] in self._ARGS_ORDER else 100)
        
        for key, value in kwargs.iteritems():
            if key == "format":
                continue # format is handled in query_url
            if key not in ARRAYEXPRESS_FIELDS:
                raise ValueError("Invalid argument name: '{0}'".format(key))
            if value is not None and value != []:
                fmt = formaters.get(key, format_default)
                value = fmt(value)
                parts.append("{0}={1}".format(key, value))
                 
        return "&".join(parts)
        
    def query_url(self, what="experiments", **kwargs):
        """ Return a formated query URL for the query arguments
        
        Example ::
            >>> conn.query_url(accession="E-MEXP-31")
            'http://www.ebi.ac.uk/arrayexpress/json/v2/experiments?accession=E-MEXP-31'
            
        """
        query = self.format_query(**kwargs)
        url = posixpath.join(self.address, what)
        url = url.format(format=kwargs.get("format", self.DEFAULT_FORMAT))
        url = url + ("?" + query if query else "")
        url = url.replace(" ", "%20")
        return url
    
    def query_url_experiments(self, **kwargs):
        """ Return a formated experiments query url for the calls arguments
        """
        return self.query_url("experiments", **kwargs)
    
    def query_url_files(self, **kwargs):
        """ Return a formated experiments query url for the calls arguments
        """
        return self.query_url("files", **kwargs)
    
    def query_experiment(self, **kwargs):
        """ Return an open stream to the experiments query results.
        
        .. note:: This member function takes the same arguments as the module
                  level `query_experiemnts` function.
          
        """
        url = self.query_url_experiments(**kwargs)
        stream = self._cache_urlopen(url, timeout=self.timeout)
        return stream
    
    def query_files(self, **kwargs):
        """ Return an open stream to the files query results.
        
        .. note:: This member function takes the same arguments as the module
                  level `query_files` function.
        """
        url = self.query_url_files(**kwargs)
        stream = self._cache_urlopen(url, timeout=self.timeout)
        return stream
    
    def open_file(self, accession, kind="raw", ext=None):
        """ Return a file handle to experiment data.
        Possible values for kind:
            - raw: return the raw data if available
            - fgem: return the processed data if available
            - biosamples: a png or svg design image
            - idf: investigation description
            - adf: array design description
            - mageml: MAGE-ML file
            
        Example ::
        
            >>> raw_file = conn.open_file("E-TABM-1087", kind="raw")
            >>> processed_file = conn.open_file("E-TABM-1087", kind="fgem")
             
        """
        stream = self.query_files(accession=accession, format="xml")
        tree = ElementTree(file=stream)
        files = tree.findall("experiment/file")
        for file in files:
            filekind = file.find("kind").text
            fileext = file.find("extension").text
            if filekind.strip() == kind and (fileext.strip() == ext or ext is None): 
                url = file.find("url").text
                return self._cache_urlopen(url.strip(), timeout=self.timeout)
            
    def _cache_urlopen(self, url, timeout=30):
        if self.cache is not None:
            with self.open_cache() as cache:
                cached = url in cache
            if not cached:
                stream = urllib2.urlopen(url, timeout=timeout)
                data = stream.read()
                with self.open_cache() as cache:
                    cache[url] = data
            else:
                with self.open_cache() as cache:
                    data = cache[url]
            return StringIO(data)
        else:
            return urllib2.urlopen(url, timeout=timeout)
        
    def open_cache(self):
        if isinstance(self.cache, basestring):
            return closing(shelve.open(self.cache))
        elif hasattr(self.cache, "close"):
            return closing(self.cache)
        elif self.cache is None:
            return fake_closing({})
        else:
            return fake_closing(self.cache)
        
from contextlib import contextmanager

@contextmanager
def fake_closing(obj):
    yield obj
    
def query_experiments(keywords=None, accession=None, array=None, ef=None,
                      efv=None, expdesign=None, exptype=None,
                      gxa=None, pmid=None, sa=None, species=None,
                      expandefo=None, directsub=None, assaycount=None,
                      efcount=None, samplecount=None, rawcount=None,
                      fgemcount=None, miamescore=None, date=None, 
                      format="json", wholewords=None, connection=None):
    """ Query Array Express experiments.
    
    :param keywords: A list of keywords to search (e.g. ['gliobastoma']
    :param accession: Search by experiment accession (e.g. 'E-MEXP-31')
    :param array: Search by array design name or accession (e.g. 'A-AFFY-33')
    :param ef: Experimental factor (names of main variables of experiments)
    :param efv: Experimental factor value (Has EFO expansion)
    :param expdesign: Experiment design type. (e.g. ["dose", "response"])
    :param exptype: Experiment type (e.g. 'RNA-Seq', has EFO expansion)
    :param gxa: If True limit the results to experiments from the Gene
        Expreission Atlas only.
    :param pmid: Search by PubMed identifier
    :param sa: Sample attribute values (e.g. 'fibroblast', has EFO expansion)
    :param species: Search by species (e.g. 'Homo sapiens', has EFO expansion)
    
    :param expandefo: If True expand the search terms with all its child terms
        in the Experimental Factor Ontology (EFO_) (e.g. keywords="cancer"
        will be expanded to include for synonyms and sub types of cancer).
    :param directsub: If True return only experiments submited directly to
        Array Express else if False return only experiments imported from GEO
        database (default None, return both)
    :param assaycount: A two tuple (min, max) for filter on the number of
        assays (e.g. (1, 5) will return only experiments with at least one
        and no more then 5 assays).
    :param efcount: Filter on the number of experimental factors (e.g. (1, 5))
    :param sacount: Filter on the number of sample attribute categories
    :param rawcount: Filter on the number or raw files
    :param fgemcount: Filter on the number of final gene expression matrix
        (processed data) files
    :param miamescore: Filter on the MIAME complience score (max 5)
    :param date: Filter by release date
    
    Example ::
    
        >>> query_experiments(species="Homo sapiens", ef="organism_part", efv="liver")
        {u'experiments': ...
        
    .. _EFO: http://www.ebi.ac.uk/efo/
    
    """
    if connection is None:
        connection = ArrayExpressConnection()
        
    stream = connection.query_experiment(keywords=keywords, accession=accession,
                array=array, ef=ef, efv=efv, expdesign=expdesign, exptype=exptype,
                gxa=gxa, pmid=pmid, sa=sa, species=species, expandefo=expandefo,
                directsub=directsub, assaycount=assaycount, efcount=efcount,
                samplecount=samplecount, rawcount=rawcount, fgemcount=fgemcount,
                miamescore=miamescore, date=date,  format=format,
                wholewords=wholewords)
    
    if format == "json":
        return parse_json(stream)
    else:
        return parse_xml(stream)
    
    
def query_files(keywords=None, accession=None, array=None, ef=None,
               efv=None, expdesign=None, exptype=None,
               gxa=None, pmid=None, sa=None, species=None,
               expandefo=None, directsub=None, assaycount=None,
               efcount=None, samplecount=None, rawcount=None,
               fgemcount=None, miamescore=None, date=None, 
               format="json", wholewords=None, connection=None):
    """ Query Array Express files.
    
    This function accepts the same arguments as `query_experiments`.
    
    Example ::
    
        >>> query_files(species="Mus musculus", ef="developmental_stage", efv="embryo", format="xml")
        <xml.etree.ElementTree.ElementTree object ...
        
    .. todo:: why does the json interface not work.
                        
    """
    if connection is None:
        connection = ArrayExpressConnection()
    
    stream = connection.query_files(keywords=keywords, accession=accession,
                array=array, ef=ef, efv=efv, expdesign=expdesign, exptype=exptype,
                gxa=gxa, pmid=pmid, sa=sa, species=species, expandefo=expandefo,
                directsub=directsub, assaycount=assaycount, efcount=efcount,
                samplecount=samplecount, rawcount=rawcount, fgemcount=fgemcount,
                miamescore=miamescore, date=date,  format=format,
                wholewords=wholewords)
    
    if format == "json":
        return parse_json(stream)
    else:
        return parse_xml(stream)
    
    
def open_file(accession, kind="raw", ext=None, repo_dir=None):
    """ Open a file for the experiment. 
     
    Example ::
    
        >>> file = open_file("E-MTAB-369", kind="raw", repo_dir="~/.arrayexpress/")
        
    """
    raise NotImplementedError

"""\
MAGE-TAB convinence functions, classes
======================================
"""

IDF_SINGLE_TAGS = \
    ["Investigation Title",
     "Date of Experiment",
     "Public Release Date",
     "Experiment Description",
    ]

def parse_idf(file):
    """ Parse an idf.txt (Investigation Description  Format) formated file.
    Return a list of tuples where the first element is the tag (first column)
    and the second element is a list of all tag values.
    
    """
    if isinstance(file, basestring):
        file = open(file, "rb")
    data = file.read()
    lines = data.splitlines()
    lines = [line.split("\t") for line in lines if line and not line.startswith("#")]
    parsed = [(line[0], line[1:]) for line in lines]
    return parsed
    
def parse_sdrf(file):
    """ Parse an sdfr formated file. Return a tuple with the first element
    a list of header values and the second element is a list of all rows
    (a row is a list of string values).
    
    """
    if isinstance(file, basestring):
        file = open(file, "rb")
    data = file.read()
    lines = data.splitlines()
    lines = [line.split("\t") for line in lines if line.strip() and not line.startswith("#")]
    header = [h for h in lines[0] if h]
    rows = [line[:len(header)] for line in lines[1:]]
    assert(all([len(r) == len(header) for r in rows]))
    return header, rows
    
def parse_adf(file):
    pass

def parse_data_matrix(file):
    """ Parse the MAGE-TAB processed data matrix. Return a tuple where the
    elements are:
        - a (header REF, header values) tuple (e.g. ("Hybridization REF", ["Stage1", "Stage2", ...]) )
        - a list of quantitations for header values (e.g. ["log2 ratio", "log2 ratio", ...])
        - a (row REF, row names list) tuple ("e.g. ("Reporter REF", ["Probe 1", "Probe 2", ...]) )
        - a list of list matrix with values (as strings)
        
    """
    if isinstance(file, basestring):
        file = open(file, "rb")
    data = file.read()
    lines = data.splitlines()
    lines = [line.split("\t") for line in lines if line.strip()]
    header = lines[0]
    header_ref, header = header[0], header[1:]
    line2 = lines[1]
    row_ref, quanifications = line2[0], line2[1:]
    row_names, rows = [], []
    for line in lines[2:]:
        r_name, row = line[0], line[1:]
        row_names.append(r_name)
        rows.append(row)
        
    return ((header_ref, header),
            quanifications,
            (row_ref, row_names),
            rows) 
    
class InvestigationDesign(dict):
    """ Investigation design (contains the contents of the .idf).
    
    Example ::
    
        >>> idf = InvestigationDesign("foobar.idf")
        >>> print idf.investigation_title
        foo investigation
        >>> print idf.experimental_design
        ['fubar', 'snafu']
        >>> print idf.sdrf_file
        ['foobar.sdrf']
        
    """
    def __init__(self, idf_file=None):
        idf = parse_idf(idf_file)
        self._idf = idf
        self.update(dict(idf))
        for tag, values in idf:
            if tag in IDF_SINGLE_TAGS:
                values = values[0] if values else None
            ttag = self.transform_tag(tag)
            setattr(self, ttag, values)
        
    def transform_tag(self, tag):
        """ Transform the tag into a proper python attribute name by
        replacing all spaces and special characters (e.g '[', ']' into
        underscores).
        
        """
        toreplace = [" ", "-", "[", "]"]
        for s in toreplace:
            tag = tag.replace(s, "_")
        return tag.lower()
            
    def __getitem__(self, tag):
        """ Return the tag values
        
        Example ::
        
            >>> idf["Investigation Title"]
            'foo investigation'
            
        """
        try:
            return self._idf_dict[tag]
        except KeyError:
            pass
        
        ttag = self.transform_tag(tag)
        if hasattr(self, ttag):
            return getattr(self, ttag)
        else:
            raise KeyError(tag)
        
class SampleDataRelationship(object):
    """ Sample-Data Relationship (contains the contents of the .sdrf file).
    
    Example ::
    
        >>> sdr = SampleDataRelationship("foobar.sdrf")
        >>> sdr.source_name
        ['foo', ...
        
        >>> sdr.sample_name
        ['sampled foo', ...
        
        >>> sdr.extract_protocol_ref
        ['bar the foo', ...
        
        >>> sdr.derived_array_data_matrix_file
        ['foobar.data.txt', ...
        
        >>>
         
    """
    
    # Nodes an edges
    NODES_EDGES = ["Source Name", "Sample Name", "Extract Name",
                   "Labeled Extract Name", "Hybridization Name",
                   "Assay Name", "Scan Name", "Normalization Name",
                   "Array Data File", "Derived Array Data File",
                   "Array Data Matrix File", "Derived Array Data Matrix File",
                   "Image File", "Protocol REF"]
    
    # Attributes for nodes and edges
    NODE_EDGE_ATTRIBUTES = \
    {"Source Name": ["Characteristics", "Provider", "Material Type", "Description", "Comment"],
     "Sample Name": ["Characteristics", "Material Type", "Description", "Comment"],
     "Extract Name":["Characteristics", "Material Type", "Description", "Comment"],
     "Labeled Extract Name": ["Characteristics", "Material Type", "Description", "Label", "Comment"],
     "Hybridization Name": ["Array Design File", "Array Design REF", "Comment"],
     "Assay Name": ["Technology Type", "Comment"],
     "Scan Name": ["Comment"],
     "Normalization Name": ["Comment"],
     "Array Data File": ["Comment"],
     "Derived Array Data File": ["Comment"],
     "Array Data Matrix File": ["Comment"],
     "Derived Array Data Matrix File": ["Comment"],
     "Image File": ["Comment"],
     "Protocol REF": ["Term Source REF", "Parameter", "Performer", "Date", "Comment"]
     }
    
    # Attributes
    ATTRIBUTE_COLUMNS = \
    {"Characteristics []": ["Unit", "Term Source REF"],
     "Provider": ["Comment"],
     "Material Type": ["Term Source REF"],
     "Label": ["Term Source REF"],
     "Array Design File": ["Comment"],
     "Array Design REF": ["Term Source REF", "Comment"],    
     "Technology Type": ["Term Source REF"],
     "Factor Value [] ()": ["Unit", "Term Source REF"],
     "Performer": ["Comment"],
     "Date": [],
     "Parameter Value []": ["Unit", "Comment"],
     "Unit []": ["Term Source REF"],
     "Description": [],
     "Term Source REF": ["Term Accession Number"],
     "Term Accession Number": [],
     "Comment []": []
     }
    def __init__(self, sdrf_file=None):
        header, rows = parse_sdrf(sdrf_file)
        self.header = header
        self.rows = rows 
        
    def transform_tag(self, tag):
        """ Transform the tag into a proper python attribute name by
        replacing all spaces and special characters (e.g '[', ']' into
        underscores).
        
        """
        toreplace = [" ", "-", "[", "]"]
        for s in toreplace:
            tag = tag.replace(s, "_")
        return tag.lower()
    
    def _subsection(self, name):
        """ Return the named subsection (name must be from the
        NODES_EDGES list).
        
        """
        idx = self.NODES_EDGES.index(name)
        start = self.header.index(name)
        end = -1
        for name in self.NODES_EDGES[idx + 1:]:
            if name in self.header[start + 1:]:
                end = self.header.index(name, start + 1)
                break
        return self.header[start:end], [r[start:end] for r in self.rows]
    
    def _column(self, name):
        """ Return the named column.
        """
        index = self.header.index(name)
        return [r[index] for r in self.rows]
        
    def source(self):
        """ Return the Source subsection
        """
        return self._subsection("Source Name")
        
    def source_name(self):
        """ Return the Source Name subsection
        """
        return self._column("Source Name")
        
    def sample(self):
        """ Return the Sample subsection
        """
        return self._subsection("Sample Name")
        
    def sample_name(self):
        """ Return the Sample Name subsection
        """
        return self._column("Sample Name")
        
    def extract(self):
        """ Return the Extract subsection
        """
        return self._subsection("Extract Name")
        
    def extract_name(self):
        """ Return the Extract Name subsection
        """
        return self._column("Extract Name")
        
    def labeled_extract(self):
        """ Return the Labeled Extract subsection
        """
        return self._subsection("Labeled Extract Name")
        
    def labeled_extract_name(self):
        """ Return the Labeled Extract Name subsection
        """
        return self._column("Labeled Extract Name")
        
    def hybridization(self):
        """ Return the Hibridization subsection.
        """
        return self._subsection("Hibridization Name")
        
    def hybridization_name(self):
        """ Return the Hibridization Name subsection.
        """
        return self._column("Hibridization Name")
        
    def assay(self):
        """ Return the Assay subsection
        """
        return self._subsection("Assay Name")
        
    def assay_name(self):
        """ Return the Assay Name subsection
        """
        return self._column("Assay Name")
        
    def scan(self):
        """ Return the Scan subsection
        """
        return self._subsection("Scan Name")
        
    def scan_name(self):
        """ Return the Scan name subsection
        """
        return self._column("Scan Name")
        
    def normalization(self):
        """ Return the Normalization subsection.
        """
        return self._subsection("Normalization Name")
        
    def normalization_name(self):
        """ Return the Normalization Name subsection.
        """
        return self._column("Normalization Name")
         
    def array_data(self):
        """ Return the Array Data subsection
        """
        return self._subsection("Array Data File")
    
    def array_data_file(self):
        """ Return the Array Data File subsection
        """
        return self._column("Array Data File")
        
    def derived_array_data(self):
        """ Return the Derived Array Data subsection
        """
        return self._subsection("Derived Array Data File")
    
    def derived_array_data_file(self):
        """ Return the Derived Array Data File subsection
        """
        return self._column("Derived Array Data File")
        
    def array_data_matrix(self):
        """ Return the Array Data Matrix subsection.
        """
        return self._subsection("Array Data Matrix File")
    
    def array_data_matrix_file(self):
        """ Return the Array Data Matrix File subsection.
        """
        return self._column("Array Data Matrix File")
        
    def derived_array_data_matrix(self):
        """ Return the Derived Array Data Matrix subsection.
        """
        return self._subsection("Derived Array Data Matrix File")
    
    def derived_array_data_matrix_file(self):
        """ Return the Derived Array Data Matrix File subsection.
        """
        return self._column("Derived Array Data Matrix File")
        
    def image(self):
        """ Return the Image subsection
        """
        return self._subsection("Image File")
    
    def image_file(self):
        """ Return the Image File subsection.
        """
        return self._column("Image File")
        
class ArrayDesign(object):
    """ Arary design (contains the contents of the .adf file).
    """
    def __init__(self, adf_file=None):
        adf = parse_adf(adf_file)
        self._adf = adf
    
def _is_float(str):
    try:
        float(str)
        return True
    except ValueError:
        return False
    
def _is_continuous(items, check_count=100):
    """ Are the strings in items continous numbers. 
    """
    count = 0
    i = 0
    for i, item in enumerate(items):
        if _is_float(item):
            count += 1
        if i >= check_count:
            break
    return count >= i * 0.5
    
def processed_matrix_to_orange(matrix_file):
    """ Load a single processed matrix file in to an Orange.data.Table
    instance. 
    """
    import numpy
    import Orange
    
    if isinstance(matrix_file, basestring):
        matrix_file = open(matrix_file, "rb")
        
#    data_matrix = matrix_file.read()
    header, quantification, rows, matrix = parse_data_matrix(matrix_file)
    header_ref, header = header
    row_ref, rows = rows
    
    matrix = numpy.array(matrix, dtype=object)
    
    
    features = []
    is_float = numpy.frompyfunc(_is_float, 1, 1) # an numpy ufunc
         
    for header_name, quant, column in zip(header, quantification, matrix.T):
        if _is_continuous(column):
            feature = Orange.feature.Continuous(header_name)
            column[numpy.where(1 - is_float(column))] = "?" # relace all non parsable floats with '?'
        else:
            values = set(column)
            feature = Orange.feature.Discrete(header_name, values=sorted(values))
        feature.attributes["quantification"] = quant
        features.append(feature)
        
    row_ref_feature = Orange.feature.String(row_ref)
    domain = Orange.data.Domain(features, None)
    domain.addmeta(Orange.feature.Descriptor.new_meta_id(), row_ref_feature)
    
    table = Orange.data.Table(domain, [list(row) for row in matrix])
    
    # Add row identifiers
    for instance, row in zip(table, rows):
        instance[row_ref_feature] = row
    
    return table
    

def mage_tab_to_orange(idf_filename):
    """ Convert an MAGE-TAB annotated experiment into an Orange.data.Table
    instance (assumes all the associated MAGE-TAB files are in the same
    directory.
    
    .. todo:: Add Characteristics, Factor Values ... to the feature.attributes dict
    
    """
    import Orange
    dirname = os.path.dirname(idf_filename)
    idf = InvestigationDesign(idf_filename)
    
    sdrf_filename = os.path.join(dirname, idf.sdrf_file[0])
    sdrf = SampleDataRelationship(sdrf_filename)
    
    data_matrices = set(sdrf.derived_array_data_matrix_file())
    data_matrices = [name for name in data_matrices if name.strip()]
    
    tables = []
    for filename in data_matrices:
        matrix_file = os.path.join(dirname, filename)
        table = processed_matrix_to_orange(matrix_file)
        tables.append(table)
        
    return hstack_tables(tables)
    
def hstack_tables(tables):
    """ Stack the tables horizontaly.
    """
    import Orange
    max_len = max([len(table) for table in tables])
    stacked_features = []
    stacked_values = [[] for i in range(max_len)]
    stacked_meta_features = []
    stacked_meta_values = [{} for i in range(max_len)]
    
    for table in tables:
        stacked_features.extend(table.domain.variables)
        stacked_meta_features.extend(table.domain.getmetas().items())
        
        for i, instance in enumerate(table):
            stacked_values[i].extend(list(instance))
            stacked_meta_values[i].update(instance.getmetas())
            
        # Fill extra lines with unknowns
        for i in range(len(table), max_len):
            stacked_values[i].extend(["?"] * len(table.domain.variables))
        
    domain = Orange.data.Domain(stacked_features, tables[-1].domain.class_var)
    domain.addmetas(dict(set(stacked_meta_features)))
    table = Orange.data.Table(domain, stacked_values)
    
    # Add meta attributes
    for instance, metas in zip(table, stacked_meta_values):
        for m, val in metas.iteritems():
            instance[m] = val
            
    return table
    
def _dictify(element):
    """ 'Dictify' and xml.etree.Element.Element instance by taking all
    subnode tags as keys and the corresponding text values as items i.e. turn
    `<node><bla>foo</bla><a>b</b></node>` into a {"bla":"foo", "a":b"}
        
    """
    if element is None:
        element = []
    dict = {}
    strip = lambda s: s.strip() if s else s
    for node in element:
        dict[node.tag] = strip(getattr(node, "text", None))
    return dict
    
class SearchableList(list):
    """ A list with a `search` method
    """
    def search(self, **kwargs):
        """ Search the list for elements with members that correspond the
        supplied keyword arguments.
        
            >>> foo.bar = "foo"
            >>> list = SearchableList([foo, bar])
            >>> list.search(bar="foo") # Search for objects which have a member named "bar" and that member equals "foo"
            [<__main__.foo object ...
            
        """
        ret = []
        for obj in self:
            for member, value in kwargs.items():
                if hasattr(obj, member) and getattr(obj, member) == value:
                    ret.append(obj)
        return ret
    
class ArrayExpressExperiment(object):
    """ An convinience class representing an Array Express Experiment.
    
    Example ::
    
        >>> ae = ArrayExpressExperiment("E-MEXP-2917")
        >>> print ae.name
        Characterization of Data Variation in Gene Expression Profiling of Human Peripheral Blood Samples
        
        >>> for file in ae.files:
        ...     print file["name"], file["url"]
        E-MEXP-2917.sdrf.txt http://www.ebi.ac.uk/arrayexpress/files/E-MEXP-2917/E-MEXP-2917.sdrf.txt
        ...
        
        >>> table = ae.fgem_to_table() # Retieve the experiment data table 
            
    """
    
    def __init__(self, accession, connection=None):
        self.accession = accession
        self.connection = connection
        self._etree = tree = query_experiments(accession=accession, connection=self.connection, format="xml")
        experiments = tree.findall("experiment")
        # find the exact match (more then one experiment can be listed in the query result)
        experiments = [e for e in experiments if e.find("accession").text.strip() == accession]
        self._experiment = experiment = experiments[0]
        
        self.species = [e.text for e in experiment.findall("species")]
        bool_values = {"true": True, "false": False}
        self.rawdatafiles = bool_values[experiment.find("rawdatafiles").get("available","false")]
        self.fgemdatafiles = bool_values[experiment.find("fgemdatafiles").get("available", "false")]
        
        self.sampleattributes = []
        for sa in experiment.findall("sampleattribute"):
            category = sa.find("category").text.strip()
            values = [val.text for val in sa.findall("value")]
            self.sampleattributes.append((category, values))
            
        self.experimentalfactors = []
        for ef in experiment.findall("experimentalfactor"):
            name = ef.find("name").text.strip()
            values = [val.text.strip() for val in ef.findall("values")]
            self.experimentalfactors.append((name, values))
            
        self.miamescores = _dictify(experiment.find("miamescores"))
            
        self.id = experiment.find("id").text
        self.secondaryaccession = getattr(experiment.find("secondaryaccession"), "text", None)
        self.name = experiment.find("name").text
        self.experimenttype = experiment.find("experimenttype").text.strip()
        self.releasedate = experiment.find("releasedate").text
        self.lastupdatedate = getattr(experiment.find("lastupdatedate"), "text", None)
        self.samples = int(experiment.find("samples").text)
        self.assays = int(experiment.find("assays").text)
        
        self.arraydesign = [_dictify(e) for e in experiment.findall("arraydesign")]
            
        self.bioassaydatagroups = [_dictify(group) for group in experiment.findall("bioassaydatagroup")]
        self.bibliography = [_dictify(e) for e in experiment.findall("bibliography")]
        self.provider = [_dictify(e) for e in experiment.findall("provider")]
        
        self.experimentdesign = []
        for expd in experiment.findall("experimentdesign"):
            self.experimentdesign.append(expd.text)
            
        self.description = [_dictify(e) for e in experiment.findall("description")]
        
        tree = query_files(accession=self.accession, format="xml", connection=self.connection)
        experiments = tree.findall("experiment")
        experiments = [e for e in experiments if e.find("accession").text.strip() == accession]
        experiment = experiments[0]
        files = experiment.findall("file")
        self.files = [_dictify(file) for file in files]
        
    def _download_processed(self):
        """ Download the processed matrix file, and associated MAGE-TAB files (idf, sdrf, adf)
        
        .. todo:: What about the raw data files (we need converters for other file types) 
        """
        assert(self.fgemdatafiles)
        exp_files = [(f["kind"], f) for f in self.files if f.get("kind") in ["idf", "sdrf"] and f.get("extension") == "txt"]
        exp_files += [(f["kind"], f) for f in self.files if f.get("kind") == "fgem"]
        array_files = [(f["kind"], f) for f in self.files if f.get("kind") == "adf" and f.get("extension") == "txt"]
        assert(len(files) == 3)
        
        for type, file in files.iteritems():
            url = file["url"].strip()
            rest, basename = os.path.split(url)
            _, dirname = os.path.split(rest)
            
            repo_dir = orngServerFiles.localpath("ArrayExpress", dirname)
            try:
                os.makedirs(repo_dir)
            except OSError:
                pass
            local_filename = os.path.join(repo_dir, basename)
            stream = urllib2.urlopen(url)
            shutil.copyfileobj(stream, open(local_filename, "wb"))
            
            if file["extension"] == "zip":
                import zipfile
                zfile = zlib.ZipFile(local_filename)
                zfile.extractall(repo_dir)
            elif file["extension"] == "gz":
                import gzip
                gzfile = gzip.open(local_filename)
                gzfile.extractall(repo_dir)
            elif file["extension"] in ["tgz", "tar"]:
                import tarfile
                tfile = tarfile.TarFile(local_filename)
                tfile.extractall(repo_dir)
            elif file["extension"] == "txt":
                pass
            else:
                raise ValueError("Unknown extension ('{0}').".format(basename))
            
    def _download_file(self, url, extract=True):
        """ Download the `file` from the ArrayExpress saving it to a local
        repository directory.
         
        """
        rest, basename = posixpath.split(url)
        dirname = posixpath.basename(rest)
        repo_dir = orngServerFiles.localpath("ArrayExpress", dirname)
        try:
            os.makedirs(repo_dir)
        except OSError:
            pass
        stream = urllib2.urlopen(url)
        local_filename = os.path.join(repo_dir, basename)
        shutil.copyfileobj(stream, open(local_filename, "wb"))
        
        if extract:
            _, extension = os.path.splitext(local_filename)
            if extension == ".zip":
                import zipfile
                zfile = zipfile.ZipFile(local_filename)
                zfile.extractall(repo_dir)
            elif extension == ".gz":
                import gzip
                gzfile = gzip.open(local_filename)
                gzfile.extractall(repo_dir)
            elif extension in [".tgz"]:
                import tarfile
                tfile = tarfile.TarFile(local_filename)
                tfile.extractall(repo_dir)
            elif extension == ".txt":
                pass
            else:
                raise ValueError("Unknown extension ('{0}').".format(basename))
            
    def _is_local(self, url):
        """ Is the `url` stored in the local repository.
        """
        return os.path.exists(self._local_filepath(url))
    
    def _local_filepath(self, url):
        """ Return the local file path for url.
        """
        rest, basename = posixpath.split(url)
        dirname = posixpath.basename(rest)
        return orngServerFiles.localpath("ArrayExpress", os.path.join(dirname, basename))
    
    def _open(self, url):
        """ Return an open file like handle to url (ArrayExpress file).
        The file is cached in the local repository for future access.
        
        """
        if not self._is_local(url):
            self._download_file(url, extract=True)
        file = self._local_filepath(url)
        return open(file, "rb")
    
    def _search_files(self, kind=None, extension=None):
        """ Search files by `kind` and `extension`.
        """
        res = []
        for file in self.files:
            kind_match = kind == file.get("kind") or kind is None
            extension_match = extension == file.get("extension") or extension is None
            
            if kind_match and extension_match:
                res.append(file)
        return res
        
    def array_design(self):
        """ Return a list of `ArrayDesign` instances used in this experiment.
        """
        files = [f for f in self.files if f.get("kind") == "adf" and \
                 f.get("extension") == "txt"]
        
        array_design = []
        for file in files:
            url = file.get("url")
            if not self._is_local(url):
                self._download_file(url)
            array_design.append(ArrayDesign(self._open(url)))
        return array_design
        
    def investigation_design(self):
        """ Return an `InvestigationDesgin` instance for this experiment
        """
        files = [f for f in self.files if f.get("kind") == "idf" and \
                 f.get("extension") == "txt"]
        if not files:
            raise ValueError("The experiment '{0}' does not have an investigation design file".format(self.accession))
        file = files[0]
        return InvestigationDesign(self._open(file.get("url")))
        
        
    def sample_data_relationship(self):
        """ Return an `SampleDataRelationship` instance describing this experiment.
        """
        files = [f for f in self.files if f.get("kind") == "sdrf" and \
                 f.get("extension") == "txt"]
        if not files:
            raise ValueError("The experiment '{0}' does not have an sample and data relationship file".format(self.accession))
        file = files[0]
        return SampleDataRelationship(self._open(file.get("url")))
        
    def fgem_to_table(self):
        """ Retrieve the processed matrix from the Array Express ftp
        server and convert it to an :class:`Orange.data.Table` instance.
         
        """
        assert(self.fgemdatafiles)
        repo_dir = orngServerFiles.localpath("ArrayExpress", self.accession)
        # Find the file listing the data matrix files (should be in sdrf but sometimes it is in 2column file only, why?)
        sdrf = self._search_files("sdrf", "txt")
        if sdrf:
            sdrf = SampleDataRelationship(self._open(sdrf[0].get("url")))
            if "Derived Array Data Matrix File" not in sdrf.header:
                twocol = self._search_files("twocolumn", "txt")
                if twocol:
                    sdrf = SampleDataRelationship(self._open(twocol[0].get("url")))
        matrix_file = self._search_files("fgem")[0]
        self._open(matrix_file.get("url")) 
        matrix_files = sorted(set(sdrf.derived_array_data_matrix_file()))
        
        idf_file = self._search_files("idf", "txt")[0]
        self._open(idf_file.get("url")) # To download if not cached
        return mage_tab_to_orange(os.path.join(repo_dir, idf_file.get("name")))
        
    
"""\
Gene Expression Atlas
---------------------

.. WARNING:: Deprecated, use ``obiGeneAtlas`` instead.

`Gene Expression Atlas <http://www.ebi.ac.uk/gxa/>`_ is a curated subset of 
gene expression experiments in Array Express Archive.

Use `query_atlas_simple` for simple querys.

Example (query human genes for experiments in which they are up regulated) ::

    >>> obiArrayExpress.query_atlas_simple(genes=["SORL1", "PSIP1", "CDKN1C"], regulation="up", organism="Homo sapiens")
    {u'...
    
Or use the `AtlasCondition` subclasses in this module to construct a more
advanced query and use the `query_atlas` function.

Example (query human genes annotated to the GO term 'transporter activity'
that are up regulated in the liver in at least three experiments) ::

    >>> go_cond = AtlasConditionGeneProperty("Goterm", "Is", "transporter activity")
    >>> liver_cond = AtlasConditionExperimentalFactor("Organism_part", "up", 3, "liver")
    >>> org_cond = AtlasConditionOrganism("Homo sapiens")
    >>> cond_list = AtlasConditionList([go_cond, liver_cond, org_cond])
    >>> query_atlas(cond_list)
    {u'...
    
"""

class GeneExpressionAtlasConenction(object):
    """ A connection to Gene Expression Atlas database.
    """
    DEFAULT_ADDRESS = "http://www.ebi.ac.uk:80/gxa/"
    try:
        DEFAULT_CACHE = shelve.open(orngServerFiles.localpath("ArrayExpress", "GeneAtlasCache.shelve"))
    except Exception, ex:
        print ex
        DEFAULT_CACHE = {}
    def __init__(self, address=None, timeout=30, cache=None):
        """ Initialize the conenction.
        
        :param address: Address of the server.
        :param timeout: Socket timeout.
        :param cache : A dict like object to use as a cache.
        
        """
        self.address = address if address is not None else self.DEFAULT_ADDRESS
        self.timeout = timeout
        self.cache = cache if cache is not None else self.DEFAULT_CACHE
    
    def query(self, condition, format="json", start=None, rows=None, indent=False):
        url = self.address + "api/vx?" + condition.rest()
        if start is not None and rows is not None:
            url += "&start={0}&rows={1}".format(start, rows)
        url += "&format={0}".format(format)
        if indent:
            url += "&indent"
#        print url
        if self.cache is not None:
            return self._query_cached(url)
        else:
            return urllib2.urlopen(url)
        return response
    
    def _query_cached(self, url):
        if self.cache is not None:
            if url not in self.cache:
                response = urllib2.urlopen(url)
                contents = response.read()
                self.cache[url] = contents
                if hasattr(self.cache, "sync"):
                    self.cache.sync()
            return StringIO(self.cache[url])
        else:
            return urllib2.urlopen(url)
                
    
# Names of all Gene Property filter names
GENE_FILTERS = \
    ["Name", # Gene name
     "Goterm", #Gene Ontology Term
     "Interproterm", #InterPro Term
     "Disease", #Gene-Disease Assocation
     "Keyword", #Gene Keyword
     "Protein", #Protein

     "Dbxref", #Other Database Cross-Refs
     "Embl", #EMBL-Bank ID
     "Ensfamily", #Ensembl Family
     "Ensgene", #Ensembl Gene ID

     "Ensprotein", #Ensembl Protein ID
     "Enstranscript", #Ensembl Transcript ID
     "Goid", #Gene Ontology ID
     "Image", #IMAGE ID
     "Interproid", #InterPro ID
     "Locuslink", #Entrez Gene ID

     "Omimid", #OMIM ID
     "Orf", #ORF
     "Refseq", #RefSeq ID
     "Unigene", #UniGene ID
     "Uniprot", #UniProt Accession

     "Hmdb", #HMDB ID
     "Chebi", #ChEBI ID
     "Cas", #CAS
     "Uniprotmetenz", #Uniprotmetenz
     "Gene", #Gene Name or Identifier
     "Synonym", #Gene Synonym
     ]
    
# Valid Gene Property filter qualifiers 
GENE_FILTER_QUALIFIERS =\
    ["Is",
     "IsNot"
     ]

# Organisms in the Atlas
ATLAS_ORGANISMS = \
    ["Anopheles gambiae",
     "Arabidopsis thaliana",
     "Bos taurus",
     "Caenorhabditis elegans",
     "Danio rerio",
     "Drosophila melanogaster",
     "Epstein barr virus",
     "Gallus gallus",
     "Homo sapiens",
     "Human cytomegalovirus",
     "Kaposi sarcoma-associated herpesvirus",
     "Mus musculus",
     "Rattus norvegicus",
     "Saccharomyces cerevisiae",
     "Schizosaccharomyces pombe",
#     "Unknown",
     "Xenopus laevis"
     ]
    
#_COMMON_TAXIDS = \
#    {"Anopheles gambiae",
#     "Arabidopsis thaliana",
#     "Bos taurus",
#     "Caenorhabditis elegans",
#     "Danio rerio",
#     "Drosophila melanogaster",
#     "Epstein barr virus",
#     "Gallus gallus",
#     "Homo sapiens",
#     "Human cytomegalovirus",
#     "Kaposi sarcoma-associated herpesvirus",
#     "Mus musculus",
#     "Rattus norvegicus",
#     "Saccharomyces cerevisiae",
#     "Schizosaccharomyces pombe",
##     "Unknown",
#     "Xenopus laevis"
#     }
    
def ef_ontology():
    """ Return the `EF <http://www.ebi.ac.uk/efo/>`_ (Experimental Factor) ontology
    """
    from . import obiOntology
#    return obiOntology.OBOOntology(urllib2.urlopen("http://efo.svn.sourceforge.net/svnroot/efo/trunk/src/efoinobo/efo.obo"))
    from Orange.orng import orngServerFiles
    # Should this be in the OBOFoundry (Ontology) domain
    try:
        file = open(orngServerFiles.localpath_download("ArrayExpress", "efo.obo"), "rb")
    except urllib2.HTTPError:
        file = urllib2.urlopen("http://efo.svn.sourceforge.net/svnroot/efo/trunk/src/efoinobo/efo.obo")
    return obiOntology.OBOOntology(file)


class AtlasCondition(object):
    """ Base class for Gene Expression Atlas query condition
    """
    def validate(self):
        """ Validate condition in a subclass.
        """
        raise NotImplementedError
    
    def rest(self):
        """ Return a REST query part in a subclass.
        """
        raise NotImplementedError
    
    
class AtlasConditionList(list, AtlasCondition):
    """ A list of AtlasCondition instances.
    """ 
    def validate(self):
        for item in self:
            item.validate()
        
    def rest(self):
        return "&".join(cond.rest() for cond in self)


class AtlasConditionGeneProperty(AtlasCondition):
    """ An atlas gene filter condition.
    
    :param property: Property of the gene. If None or "" all properties 
        will be searched.
    :param qualifier: Qualifier can be 'Is' or 'IsNot'
    :param value: The value to search for.
    
    Example ::
    
        >>> # Condition on a gene name
        >>> condition = AtlasConditionGeneProperty("Name", "Is", "AS3MT")
        >>> # Condition on genes from a GO Term
        >>> condition = AtlasConditionGeneProperty("Goterm", "Is", "p53 binding")
        >>> # Condition on disease association
        >>> condition = AtlasConditionGeneProperty("Disease", "Is", "cancer")
        
    """
    def __init__(self, property, qualifier, value):
        self.property = property or ""
        self.qualifier = qualifier
        if isinstance(value, basestring):
            self.value = value.replace(" ", "+")
        elif isinstance(value, list):
            self.value = "+".join(value)
        else:
            raise ValueError(value)
        
        self.validate()
        
    def validate(self):
        assert(self.property in GENE_FILTERS + [""])
        assert(self.qualifier in GENE_FILTER_QUALIFIERS + [""])
        
    def rest(self):
        return "gene{property}{qualifier}={value}".format(**self.__dict__)
        
        
class AtlasConditionExperimentalFactor(AtlasCondition):
    """ An atlas experimental factor filter condition.
    
    :param factor: EFO experiamntal factor
    :param regulation: "up", "down", "updown", "any" or "none"
    :param n: Minimum number of of experimants with this condition
    :param value: Experimantal factor value
    
    Example ::
    
        >>> # Any genes up regulated in at least 3 experiments involving cancer.
        >>> condition = AtlasConditionExperimentalFactor("", "up", 3, "cancer")
        >>> # Only genes which are up/down regulated in the heart in at least one experiment. 
        >>> condition = AtlasConditionExperimentalFactor("Organism_part", "updown", 1, "heart")
        
    """
    def __init__(self, factor, regulation, n, value):
        self.factor = factor
        self.regulation = regulation
        self.n = n
        self.value = value
        self.validate()
        
    def validate(self):
        # TODO: validate the factor and value
#        assert(self.factor in ef_ontology())
        assert(self.regulation in ["up", "down", "updown"])
        
    def rest(self):
        return "{regulation}{n}In{factor}={value}".format(**self.__dict__)
        
class AtlasConditionOrganism(AtlasCondition):
    """ Condition on organism.
    """
    def __init__(self, organism):
        self.organism = organism
        self.validate()
        
    def validate(self):
        assert(self.organism in ATLAS_ORGANISMS)
        
    def rest(self):
        return "species={0}".format(self.organism.replace(" ", "+").lower())
        
        
class AtlasConditionExperiment(AtlasCondition):
    """ Condition on experiement
    
    :param property: Property of the experiment. If None or "" all properties 
        will be searched.
    :param qualifier: Qualifier can be 'Has' or 'HasNot'
    :param value: The value to search for.
    
    Example ::
    
        >>> # Condition on a experiemnt acession
        >>> condition = AtlasConditionExperiment("", "", "E-GEOD-24283")
        >>> # Condition on experiments involving lung
        >>> condition = AtlasConditionExperiment("Organism_part", "Has", "lung")
        
    """
#    EXPERIMENT_FILTERS = [
#                "Organism"
#                "Factor"]
    
    EXPERIMENT_FILTER_QUALIFIERS = [
                "Has",
                "HasNot"]
    
    def __init__(self, property, qualifier, value):
        self.property = property
        self.qualifier = qualifier
        if isinstance(value, basestring):
            self.value = value.replace(" ", "+")
        elif isinstance(value, list):
            self.value = "+".join(value)
        else:
            raise ValueError(value)
        
        self.validate()
        
    def validate(self):
        # TODO: check to EFO factors
#        assert(self.property in EXPERIMENT_FILTERS + [""])
        assert(self.qualifier in self.EXPERIMENT_FILTER_QUALIFIERS + [""])
        
    def rest(self):
        return "experiment{property}{qualifier}={value}".format(**self.__dict__)
        
        
class GeneAtlasError(ValueError):
    """ An error response from the Atlas server.
    """
    pass
    
    
def __check_atlas_error_json(response):
    if "error" in response:
        raise GeneAtlasError(response["error"])
    return response
 
     
def __check_atlas_error_xml(response):
    error = response.find("error")
    if error is not None:
        raise GeneAtlasError(error.text)
    return response
    
        
def query_atlas_simple(genes=None, regulation=None, organism=None,
                       condition=None, format="json", start=None,
                       rows=None):
    """ A simple Atlas query.
    
    :param genes: A list of gene names to search for.
    :param regulation: Search for experiments in which `genes` are "up",
        "down", "updown" or "none" regulated. If None all experiments
        are searched.
    :param organism: Search experiments for organism. If None all experiments
        are searched.
    :param condition: An EFO factor value (e.g. "brain")
    
    Example ::
        
        >>> query_atlas_simple(genes=['Pou5f1', 'Dppa3'], organism="Mus musculus")
        {u'...
        
        >>> query_atlas_simple(genes=['Pou5f1', 'Dppa3'], regulation="up", organism="Mus musculus")
        {u'...
        
        >>> query_atlas_simple(genes=['Pou5f1', 'Dppa3'], regulation="up", condition="liver", organism="Mus musculus")
        {u'...
        
    """
    import warnings
    warnings.warn("Use 'obiGeneAtlas.run_simple_query' instead.", DeprecationWarning)
    conditions = AtlasConditionList()
    if genes:
        conditions.append(AtlasConditionGeneProperty("Gene", "Is", genes))
    if regulation or condition:
        regulation = "any" if regulation is None else regulation
        condition = "" if condition is None else condition
        conditions.append(AtlasConditionExperimentalFactor("", regulation, 1, condition))
    if organism:
        conditions.append(AtlasConditionOrganism(organism))
        
    connection = GeneExpressionAtlasConenction()
    results = connection.query(conditions, format=format, start=start,
                               rows=rows)
    if format == "json":
        return parse_json(results)
    else:
        return parse_xml(results)

"""\
.. todo:: can this be implemented query_atlas(organism="...", Locuslink="...", Chebi="...", up3InCompound="..." downInEFO="...")
      Need a full list of accepted factors 
"""

def query_atlas(condition, format="json", start=None, rows=None, indent=False, connection=None):
    """ Query Atlas based on a `condition` (instance of AtlasCondition)
    
    Example ::
        
        >>> condition1 = AtlasConditionGeneProperty("Goterm", "Is", "p53 binding")
        >>> condition2 = AtlasConditionExperimentalFactor("Organism_part", "up", 3, "heart")
        >>> condition = AtlasConditionList([condition1, condition2])
        >>> query_atlas(condition)
        {u'...
        
    """
    import warnings
    warnings.warn("Use 'obiGeneAtlas.run_query' instead.", DeprecationWarning)
    if connection is None:
        connection = GeneExpressionAtlasConenction()
    results = connection.query(condition, format=format, start=start,
                               rows=rows, indent=indent)
    if format == "json":
        response = parse_json(results)
        return __check_atlas_error_json(response)
    else:
        response = parse_xml(results)
        return __check_atlas_error_xml(response)


def get_atlas_summary(genes, organism, connection=None):
    """ Return 3 dictionaries containing a summary of atlas information
    about three experimental factors:
    
        - Organism Part (OP) 
        - Disease State (DS)
        - Cell type (CT)
    
    Each dictionary contains query genes as keys. Values are dictionaries
    mapping factor values to a 2-tuple containig the count of up regulated
    and down regulated experiments.
    
    Example ::
    
        >>> get_atlas_summary(["RUNX1"], "Homo sapiens")
        ({u'RUNX1': ...
        
    """
    import warnings
    warnings.warn("Use 'obiGeneAtlas.get_atlas_summary' instead.", DeprecationWarning)
    genes_condition = AtlasConditionGeneProperty("Gene", "Is", genes)
    org_condition = AtlasConditionOrganism(organism)
    condition = AtlasConditionList([genes_condition, org_condition])
    result = query_atlas(condition, format="json", connection=connection)
    
    org_part = collect_ef_summary(result, "organism_part")
    disease_state = collect_ef_summary(result, "disease_state")
    cell_type = collect_ef_summary(result, "cell_type")
    
    return dict(org_part), dict(disease_state), dict(cell_type)
    
    
def collect_ef_summary(info, ef, summary=None):
    """ Collect the results summary from query_atlas, result for experimental
    factor `ef`. 
    """
    if summary is None:
        summary = defaultdict(dict)
        
    results = info["results"]
    for res in results:
        gene = res["gene"]
        expressions = res["expressions"] 
        for expression in expressions:
            if expression["ef"] == ef:
                efv = expression["efv"]
                updown = (expression["upExperiments"],
                          expression["downExperiments"]
                          )
                
                if any(updown):
                    summary[gene["name"]][efv] = updown
    
    return summary
    
    
def test():    
    conn = ArrayExpressConnection()
    import doctest
    foo = type("foo", (object,), {})()
    bar = type("bar", (object,), {})()
    
    doctest.testmod(optionflags=doctest.ELLIPSIS,
                    extraglobs={"conn": conn, "foo": foo,"bar": bar})
    
    
if __name__ == "__main__":
    test()
    



from __future__ import absolute_import, with_statement

import ftplib
import urllib, urllib2
import threading
import os, sys
import time
import socket
from Queue import Queue
from StringIO import StringIO
from datetime import datetime
from threading import RLock

from .obiGenomicsUpdate import synchronized

class FileNotFoundError(IOError):
    pass

class SharedCache(object):
    def __init__(self, *args, **kwargs):
        self._dict = dict(*args, **kwargs)
        self._lock = RLock()

    def __getitem__(self, key):
        with self._lock:
            return self._dict.__getitem__(key)

    def __setitem__(self, key, item):
        with self._lock:
            return self._dict.__setitem__(key, item)

    def __delitem__(self, key):
        with self._lock:
            return self._dict.__delitem__(key)

    def __contains__(self, key):
        with self._lock:
            return self._dict.__contains__(key)

    def __getattr__(self, name):
        try:
            attr = getattr(self._dict, name)
            if callable(attr):
                return synchronized(self._lock)(attr)
            else:
                return attr
        except AttributeError:
            raise AttributeError(name)
        
class _ftpCallbackWrapper(object):
    def __init__(self, size, callback, progressCallback):
        self.size = max(size, 1)
        self.callback = callback
        self.progressCallback = progressCallback
        self.transferCount = 0

    def __call__(self, block):
        self.callback(block)
        self.progressCallback(min(int(100.0*self.transferCount/self.size), 100))
        self.transferCount+=len(block)

_monthDict = {"Jan":1, "Feb":2, "Mar":3, "Apr":4, "May":5, "Jun":6, "Jul":7, "Aug":8, "Sep":9, "Oct":10, "Nov":11, "Dec":12}

class FtpWorker(object):
    def __init__(self,  ftpAddr, statCache = None):
        self.ftpAddr = ftpAddr
        self.ftp = None
        self.now = datetime.now()
        self.statCache = statCache if statCache != None else SharedCache()

    def connect(self):
        if not self.ftp:
            self.ftp = ftplib.FTP()
        self.ftp.connect(self.ftpAddr)
        self.ftp.sock.settimeout(30)
        self.ftp.login()
        
    def retrieve(self, filename, local, update, progressCallback=None):
        local = os.path.normpath(local)
        isLocal = self.isLocal(local)
        if not update and isLocal:
            return
        retryCount = 0
        while retryCount<3:
            if not self.ftp:
                self.ftp = ftplib.FTP()
                self.connect()
            try:
                retryCount+=1
                size, date = self.statFtp(filename)
                #if update and update!="force":
                if isLocal:
                    sizeLocal, dateLocal = self.statLocal(local)
                    if sizeLocal!=size:
                        update = "force"
                    #print dateLocal, date, dateLocal < date
                    if dateLocal < date: # TODO fix date comparison
                        update = "force"
                if update=="force" or not isLocal:
                    s = StringIO()
##                    f = open(local + ".tmp", "wb")
                    if progressCallback:
                        self.ftp.retrbinary("RETR "+filename, _ftpCallbackWrapper(size, s.write, progressCallback), )
                    else:
                        self.ftp.retrbinary("RETR "+filename, s.write)
                    s.getvalue()
                    if s.len>size:
                        raise Exception("Wrong size of file "+filename)
                    f = open(local, "wb")
                    f.write(s.buf)
                    f.flush()
                    f.close()
##                    try:
##                        if os.path.exists(local):
##                            os.remove(local)
##                        os.rename(local + ".tmp", local)
##                    except Exception, ex:
##                        print ex, local
##                        raise
                    break
            except ftplib.error_perm, ex:
                if ex.args[0].startswith("550"):
                    self.connect()
                else:
                    raise
            except ftplib.error_temp, ex:
                if retryCount >= 3:
                    raise
                else:
                    time.sleep(3)
            except socket.error:
                if retryCount >= 3:
                    raise
                else:
                    self.connect()
            except FileNotFoundError:
                raise
    
    def isLocal(self, filename):
        try:
            open(filename)
            return True
        except:
            return False
        
    def statFtp(self, filename):
        if not self.ftp:
            self.connect()
        dir, file = os.path.split(filename)
        dir = dir + "/"
        with self.statCache._lock:
            if dir in self.statCache:
                if file:
                    try:
                        s = self.statCache[dir][file].split()
                    except KeyError:
                        raise FileNotFoundError(filename)
                else:
                    return
            else:
                lines = []
##                print "Ftp Stat:", dir, file
                self.ftp.dir(dir, lines.append)
                self.statCache[dir] = dict([(line.split()[-1].strip(), line.strip()) for line in lines if line.strip()])
                if file:
                    try:
                        s = self.statCache[dir][file].split()
                    except KeyError:
                        raise FileNotFoundError(filename)
                else:
                    return
##                print dir ,file, s
    ##            s = s.getvalue().split()
        size, date = int(s[-5]), s[-4:-1]
        if ":" in date[-1]:
            date = datetime(self.now.year, _monthDict.get(date[0], 1), int(date[1]))
            if date>self.now:
                date = datetime(date.year-1, date.month, date.day)
        else:
            date = datetime(int(date[-1]), _monthDict.get(date[0], 1), int(date[1]))
        return size, date

    def statLocal(self, filename):
        stat = os.stat(filename)
        return stat.st_size, datetime.fromtimestamp(stat.st_mtime)
    
    def listdir(self, ftp_dir):
        """ List the contents of a remote ftp directory (similar to os.listdir)
        """
        if not self.ftp:
            self.connect()
        lines = []
        self.ftp.dir(ftp_dir, lines.append)
        self.statCache[dir] = dict([(line.split()[-1].strip(), line.strip()) for line in lines if line.strip()])
        contents = [line.split()[-1] for line in lines]
        return [name for name in contents if name not in [".", ".."]] 
        

class FtpThreadWorker(threading.Thread, FtpWorker):
    def __init__(self, ftpAddr, queue, statCache=None, group=None, target=None, name=None, args=(), kwargs={}):
        threading.Thread.__init__(self, group, target, name, args, kwargs)
        FtpWorker.__init__(self, ftpAddr, statCache)
        self.queue = queue

    def run(self):
        while True:
            filename, local, update, retryCount, progressCallback = self.queue.get()
            try:
                self.retrieve(filename, local, update, progressCallback)
            except Exception, ex:
                sys.excepthook(*sys.exc_info())
            self.queue.task_done()

class FtpDownloader(object):
    def __init__(self, ftpAddr, localDir, ftpDir="", numOfThreads=5):
        self.ftpAddr = ftpAddr
        self.localDir = localDir
        self.ftpDir = ftpDir
        self.numOfThreads = numOfThreads
        self.queue = Queue(0)
        self.workers = []
        self.statCache = SharedCache()
        self.ftpWorker = FtpWorker(self.ftpAddr, statCache=self.statCache)

    def initWorkers(self):
        if self.workers:
            return
        for i in range(self.numOfThreads):
            try:
                t = FtpThreadWorker(self.ftpAddr, self.queue, self.statCache, name="FtpThread(%s):#%i"%(self.ftpAddr, i))
                t.setDaemon(True)
                t.start()
                self.workers.append(t)
            except Exception, ex:
                print ex
                print "Cannot create thread num: "+str(i)
                
    def massRetrieve(self, filenames, update=False, blocking=True, progressCallback=None):
        self.initWorkers()
        for filename in filenames:
            self.retrieve(filename, update, blocking=False)
##            localDir = os.path.split(self.localDir+filename)[0]
##            try:
##                os.makedirs(localDir)
##            except:
##                pass
##            self.queue.put((self.ftpDir+filename, self.localDir+filename, update, 0, None))
        if blocking:
            self.wait(progressCallback)
            
    def retrieve(self, filename, update=False, blocking=True, progressCallback=None):
        if type(filename) == str:
            filename = (filename, filename)
        localDir = os.path.split(os.path.join(self.localDir, filename[1]))[0]
        try:
            os.makedirs(localDir)
        except:
            pass
        if blocking:
            self.ftpWorker.retrieve(self.ftpDir+filename[0], os.path.join(self.localDir, filename[1]), update, progressCallback)
        else:
            self.queue.put((self.ftpDir+filename[0], os.path.join(self.localDir, filename[1]), update, 0, progressCallback))

    def wait(self, progressCallback=None):
        count = self.queue.qsize()
        while not self.queue.empty():
            if progressCallback:
                progressCallback(min(100.0, 100.0*(float(count)-self.queue.qsize())/count))
            time.sleep(0.1)
        self.queue.join()
        
    def listdir(self, ftp_dir):
        """ List the contents of the remote ftp dir (similar to os.listdir)
        """
        return self.ftpWorker.listdir(self.ftpDir + ftp_dir)
    
